Package: greenbook
Title: HM Treasury Green Book Cost-Benefit Analysis Primitives
Version: 0.1.0
Authors@R: 
    person("Charles", "Coverdale", , "charles.f.coverdale@gmail.com", role = c("aut", "cre"))
Description: Implements cost-benefit analysis primitives from HM
    Treasury Green Book guidance (HM Treasury, 2022, 2026):
    the kinked Social Time Preference Rate ('STPR'), discount factors,
    net present value ('NPV'), equivalent annual cost, and real-terms
    rebasing using the GDP deflator. Designed for UK central government
    appraisal and evaluation. Bundled parameter tables carry vintage
    metadata for reproducibility.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Imports: cli (>= 3.6.0), stats, utils
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, inflateR, openxlsx,
        officer, flextable
Config/testthat/edition: 3
URL: https://github.com/charlescoverdale/greenbook
BugReports: https://github.com/charlescoverdale/greenbook/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-26 13:10:38 UTC; charlescoverdale
Author: Charles Coverdale [aut, cre]
Maintainer: Charles Coverdale <charles.f.coverdale@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 19:40:18 UTC
Built: R 4.5.2; ; 2026-05-01 17:11:37 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(format,gb_appraisal)
S3method(print,gb_appraisal)
S3method(print,gb_comparison)
S3method(print,gb_economic_case)
S3method(print,gb_headline)
S3method(print,gb_place_based)
S3method(print,gb_progression)
S3method(print,gb_risk_register)
S3method(print,gb_sensitivity_ob)
S3method(print,gb_validation)
S3method(summary,gb_appraisal)
S3method(summary,gb_comparison)
export(gb_apply_ob)
export(gb_appraise)
export(gb_carbon_npv)
export(gb_carbon_value)
export(gb_categories)
export(gb_compare)
export(gb_cost_per_unit)
export(gb_data_versions)
export(gb_deflator)
export(gb_discount)
export(gb_discount_factor)
export(gb_dist_weight)
export(gb_dist_weighted_npv)
export(gb_eanc)
export(gb_economic_case)
export(gb_headline)
export(gb_metb)
export(gb_npv)
export(gb_optimism_bias)
export(gb_place_based)
export(gb_progression)
export(gb_qaly)
export(gb_real)
export(gb_rebase)
export(gb_risk_register)
export(gb_schedule_table)
export(gb_sensitivity_ob)
export(gb_stpr)
export(gb_to_excel)
export(gb_to_latex)
export(gb_to_word)
export(gb_validate)
export(gb_vpf)
export(gb_wellby)
importFrom(cli,cli_abort)
importFrom(cli,cli_h1)
importFrom(cli,cli_inform)
importFrom(cli,cli_text)
importFrom(cli,cli_warn)
importFrom(stats,median)
importFrom(utils,read.csv)

# greenbook

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/greenbook)](https://CRAN.R-project.org/package=greenbook)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![License: MIT](https://img.shields.io/badge/license-MIT-blue.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

Cost-benefit analysis primitives from the HM Treasury Green Book, in R.


## What is the Green Book?

The [Green Book](https://www.gov.uk/government/publications/the-green-book-appraisal-and-evaluation-in-central-government) is HM Treasury's guidance on how to appraise and evaluate proposals involving UK public spending. It sets the rules every central-government department, devolved administration, and arm's-length body follows when deciding whether a policy, programme, or capital project is worth funding. The latest edition was published in 2026.

The guidance covers six core areas:

- **Discounting**: the kinked Social Time Preference Rate (STPR), stepping from 3.5 percent for the first 30 years down to 1.0 percent beyond year 300.
- **Real-terms appraisal**: rebasing nominal cashflows using the GDP deflator at market prices.
- **Optimism bias**: standard uplifts on capital cost and works-duration estimates, by project category (Mott MacDonald 2002).
- **Distributional analysis**: iso-elastic weights on net benefits accruing to different income groups.
- **Adjustments**: the Marginal Excess Tax Burden (METB) on revenue raised through distortionary taxation.
- **Monetised valuation**: WELLBYs for wellbeing, Value of Preventing a Fatality (VPF) for life-safety, Quality-Adjusted Life Years (QALYs) for health, DESNZ carbon values for emissions.

The Green Book is supplemented by topic-specific guidance from HMT, DESNZ, DfT, and DHSC.


## How is it used?

A practitioner appraising a public-spending option typically:

1. Builds a profile of costs and benefits in real terms, by year, in a fixed price base.
2. Discounts each year's cashflow under the kinked STPR to compute net present value (NPV) and benefit-cost ratio (BCR).
3. Uplifts ex-ante cost estimates by an optimism bias percentage matched to the project category.
4. Applies distributional weights if the option is regressive across income groups.
5. Runs sensitivity tests on the largest assumptions and computes switching values.

Today this is mostly done in spreadsheets, with discount factors and parameter tables hand-typed from PDFs. `greenbook` puts the same primitives in R so an appraisal becomes code that can be tested, reviewed, and reproduced.


## Why this package?

No existing R or Python package implements the Green Book. UK appraisal practitioners hand-roll the same discount factors and parameter lookups every time. The arithmetic is simple but the parameters change: STPR is kinked across six bands, optimism bias has a six-category schedule, DESNZ publishes a carbon path to 2100, METB shifted from 30 to 20 percent in 2018.

`greenbook` solves three problems:

- **Reproducibility**: every `gb_appraisal` carries vintage metadata for the parameter tables it used. `gb_data_versions()` shows source and last-updated date for every bundled table.
- **Auditability**: appraisals are code, not spreadsheets. Reviewers can run the tests, inspect the inputs, verify the outputs.
- **Maintenance**: when HM Treasury updates the Green Book, you bump the package and your existing appraisals stay aligned.

The package is pure computation: no network calls, no API keys. Bundled parameter tables in `inst/extdata/` are refreshed via `data-raw/` scripts.


## Installation

```r
# install.packages("greenbook")  # not yet on CRAN
# Development version:
devtools::install_github("charlescoverdale/greenbook")
```


## Quick start

```r
library(greenbook)

# 10-year cashflow: capex in years 0-2, benefits in years 3-9
costs    <- c(100, 50, 50, 0, 0, 0, 0, 0, 0, 0)
benefits <- c(0, 0, 0, 30, 30, 30, 30, 30, 30, 30)

# One-call full appraisal with optimism bias and METB
app <- gb_appraise(
  costs, benefits,
  ob = "non_standard_buildings", ob_mitigation = 0.5,
  metb = TRUE,
  base_year = 2024
)
summary(app)

# Equivalent annual net benefit
gb_eanc(app)

# Distributional weighting
gb_dist_weighted_npv(
  cashflow = rep(30, 5),
  recipient_income = rep(15000, 5),
  income_data = seq(10000, 100000, length.out = 10)
)

# Carbon emissions
gb_carbon_npv(rep(100, 7), 2024:2030, base_year = 2024)

# Wellbeing
gb_wellby(1, persons = 100, years = 5, base_year = 2024)

# Inspect bundled vintages
gb_data_versions()
```


## Function inventory

| Family | Functions |
|---|---|
| Discounting | `gb_stpr()`, `gb_discount_factor()`, `gb_discount()`, `gb_npv()`, `gb_eanc()` |
| Real / nominal | `gb_deflator()`, `gb_real()`, `gb_rebase()` |
| Optimism bias | `gb_optimism_bias()`, `gb_apply_ob()`, `gb_categories()` |
| Distributional | `gb_dist_weight()`, `gb_dist_weighted_npv()` |
| Valuation | `gb_wellby()`, `gb_vpf()`, `gb_qaly()` |
| Carbon | `gb_carbon_value()`, `gb_carbon_npv()` |
| Adjustments | `gb_metb()` |
| Appraisal | `gb_appraise()`, `gb_compare()`, `gb_progression()`, `gb_place_based()`, `gb_economic_case()`, `gb_validate()` |
| Risk | `gb_risk_register()` |
| Sensitivity | `gb_sensitivity_ob()` |
| Reporting | `gb_headline()`, `gb_cost_per_unit()`, `gb_to_latex()`, `gb_to_excel()`, `gb_to_word()` |
| Lookups | `gb_schedule_table()`, `gb_data_versions()` |


## Bundled data sources

Every parameter table is sourced from a published HMT, DfT, or DESNZ workbook. `gb_data_versions()` returns the full provenance.

| Table | Source |
|---|---|
| STPR schedule | HMT Green Book 2022 Annex A6 |
| GDP deflator (1990-2024, 2024 = 100) | HMT GDP deflators, December 2025 Quarterly National Accounts release |
| Optimism bias upper bounds | HMT Supplementary Green Book Guidance: Optimism Bias (Mott MacDonald 2002) |
| METB | HMT Green Book 2022 |
| WELLBY (10k / 13k / 16k, 2019 prices) | HMT Wellbeing Guidance for Appraisal, July 2021 |
| QALY (DHSC 70k / NICE 20k, 30k) | DHSC Supplementary Green Book Guidance + NICE HTA |
| VPF (anchored 2023 = GBP 2,474,341) | DfT TAG data book v2.03FC, December 2025, Table A4.1.1 (WTP element) |
| DESNZ carbon (2020-2050, 2022 prices) | DESNZ Valuation of Energy Use and GHG Emissions for Appraisal, November 2023, Table 3 |

`data-raw/` contains the refresh scripts: each downloads the published workbook, parses the relevant sheet, and rewrites the bundled CSV.

## Limitations

- VPF series interpolates between published anchors at 2 percent real GDP per head growth (TAG's stated methodology). For sub-annual precision consult the TAG data book directly.
- Bundled DESNZ carbon path covers 2020 to 2050. The November 2023 publication extends to 2100; the 2050-2100 portion will be added in a future refresh.
- Long-horizon (50+ year) lower-rate sensitivity awaits the 2026 HMT discount-rate review.


## Source documents

- [HM Treasury Green Book (2026)](https://www.gov.uk/government/publications/the-green-book-appraisal-and-evaluation-in-central-government/the-green-book-2026)
- [Supplementary Green Book Guidance: discounting](https://www.gov.uk/government/publications/green-book-supplementary-guidance-discounting)
- [Supplementary Green Book Guidance: optimism bias](https://www.gov.uk/government/publications/green-book-supplementary-guidance-optimism-bias)
- [Wellbeing Guidance for Appraisal: Supplementary Green Book Guidance, July 2021](https://www.gov.uk/government/publications/green-book-supplementary-guidance-wellbeing)
- [DESNZ Valuation of Energy Use and GHG Emissions for Appraisal, November 2023](https://www.gov.uk/government/publications/valuation-of-energy-use-and-greenhouse-gas-emissions-for-appraisal)
- [DfT Transport Analysis Guidance (TAG)](https://www.gov.uk/guidance/transport-analysis-guidance-tag)


## Citation

If you use `greenbook` in published work, please cite via:

```r
citation("greenbook")
```

The package citation and the underlying HM Treasury Green Book are both returned.


## Issues

Report bugs or request features at [GitHub Issues](https://github.com/charlescoverdale/greenbook/issues).


## Keywords

cost-benefit-analysis, appraisal, hm-treasury, green-book, public-policy, economics, discounting, npv, social-time-preference-rate, optimism-bias, distributional-weighting, wellby, value-of-statistical-life, carbon-valuation, desnz

#' Aggregate sub-projects into a place-based business case
#'
#' Combines multiple `gb_appraisal` objects into a single portfolio
#' appraisal. Introduced in the 2026 Green Book as a recognised
#' business case structure for places (city deals, devolved
#' settlements, regional packages) where individual projects are
#' interdependent.
#'
#' @param ... `gb_appraisal` objects representing sub-projects.
#'   Pass as named arguments to label each.
#' @param place Character scalar. Place name (e.g. `"Greater
#'   Manchester"`). Optional.
#' @param synergy Numeric scalar in `[-0.5, 0.5]`. Optional uplift
#'   applied to aggregate benefits to capture cross-project synergy
#'   or drag. Default `0` (additive aggregation).
#'
#' @return A `gb_place_based` object with elements `projects`,
#'   `place`, `aggregate_npv`, `aggregate_bcr`, `per_project`,
#'   `synergy`.
#'
#' @references HM Treasury (2026). The Green Book, on place-based
#'   business cases.
#'
#' @family appraisal
#' @seealso [gb_appraise()], [gb_compare()].
#'
#' @export
#' @examples
#' transport <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
#' housing <- gb_appraise(c(50, 0, 0, 0, 0), c(0, 20, 20, 20, 20))
#' skills <- gb_appraise(c(20, 20, 0, 0, 0), c(0, 0, 15, 15, 15))
#' gb_place_based(transport = transport, housing = housing,
#'                skills = skills, place = "Example City")
gb_place_based <- function(..., place = NULL, synergy = 0) {
  validate_numeric(synergy, "synergy")
  if (length(synergy) != 1L) cli::cli_abort("{.arg synergy} must be scalar.")
  if (synergy < -0.5 || synergy > 0.5) {
    cli::cli_abort("{.arg synergy} must be in [-0.5, 0.5].")
  }

  projects <- list(...)
  if (length(projects) < 1L) {
    cli::cli_abort("{.fn gb_place_based} requires at least one project.")
  }
  if (is.null(names(projects)) || any(!nzchar(names(projects)))) {
    names(projects) <- paste0("project_", seq_along(projects))
  }
  for (i in seq_along(projects)) {
    if (!inherits(projects[[i]], "gb_appraisal")) {
      cli::cli_abort("All inputs must be {.cls gb_appraisal} objects.")
    }
  }

  rows <- lapply(seq_along(projects), function(i) {
    a <- projects[[i]]
    data.frame(
      project = names(projects)[i],
      npv = a$npv,
      bcr = a$bcr %||% NA_real_,
      pv_costs = a$pv_costs %||% NA_real_,
      pv_benefits = a$pv_benefits %||% NA_real_,
      stringsAsFactors = FALSE
    )
  })
  tbl <- do.call(rbind, rows)

  total_pv_costs <- sum(tbl$pv_costs, na.rm = TRUE)
  total_pv_benefits <- sum(tbl$pv_benefits, na.rm = TRUE) * (1 + synergy)
  aggregate_npv <- total_pv_benefits - total_pv_costs
  aggregate_bcr <- if (total_pv_costs > 0) total_pv_benefits / total_pv_costs else NA_real_

  out <- list(
    projects = projects,
    place = place,
    aggregate_npv = aggregate_npv,
    aggregate_bcr = aggregate_bcr,
    per_project = tbl,
    synergy = synergy
  )
  class(out) <- c("gb_place_based", "list")
  out
}

#' @export
print.gb_place_based <- function(x, ...) {
  npv_str <- .format_gbp(x$aggregate_npv)
  bcr_str <- sprintf("%.2f", x$aggregate_bcr)
  syn_str <- sprintf("%+.0f%%", 100 * x$synergy)
  n_proj <- nrow(x$per_project)
  cli::cli_h1("Place-based business case")
  if (!is.null(x$place)) cli::cli_text("Place: {.field {x$place}}")
  cli::cli_text("Projects: {.val {n_proj}}")
  cli::cli_text("Aggregate NPV: {.val {npv_str}}")
  if (!is.na(x$aggregate_bcr)) {
    cli::cli_text("Aggregate BCR: {.val {bcr_str}}")
  }
  if (x$synergy != 0) {
    cli::cli_text("Synergy uplift: {.val {syn_str}}")
  }
  cat("\n")
  print(x$per_project, row.names = FALSE)
  invisible(x)
}

test_that("gb_place_based aggregates project NPVs additively when no synergy", {
  a <- gb_appraise(c(50, 0), c(0, 100))
  b <- gb_appraise(c(50, 0), c(0, 80))
  pb <- gb_place_based(a = a, b = b)
  expected <- a$npv + b$npv
  expect_equal(pb$aggregate_npv, expected, tolerance = 1e-9)
})

test_that("gb_place_based applies synergy uplift to benefits", {
  a <- gb_appraise(c(50, 0), c(0, 100))
  b <- gb_appraise(c(50, 0), c(0, 80))
  pb_no <- gb_place_based(a = a, b = b, synergy = 0)
  pb_up <- gb_place_based(a = a, b = b, synergy = 0.1)
  expect_gt(pb_up$aggregate_npv, pb_no$aggregate_npv)
})

test_that("gb_place_based stores place name", {
  a <- gb_appraise(c(50, 0), c(0, 100))
  b <- gb_appraise(c(50, 0), c(0, 80))
  pb <- gb_place_based(a = a, b = b, place = "Manchester")
  expect_equal(pb$place, "Manchester")
})

test_that("gb_place_based errors on out-of-range synergy", {
  a <- gb_appraise(c(50, 0), c(0, 100))
  b <- gb_appraise(c(50, 0), c(0, 80))
  expect_error(gb_place_based(a, b, synergy = 0.6))
  expect_error(gb_place_based(a, b, synergy = -0.6))
})

test_that("gb_place_based per-project table has all rows", {
  a <- gb_appraise(c(50, 0), c(0, 100))
  b <- gb_appraise(c(50, 0), c(0, 80))
  c <- gb_appraise(c(20, 20), c(0, 50))
  pb <- gb_place_based(a = a, b = b, c = c)
  expect_equal(nrow(pb$per_project), 3L)
})

test_that("gb_place_based print method does not error", {
  a <- gb_appraise(c(50, 0), c(0, 100))
  b <- gb_appraise(c(50, 0), c(0, 80))
  expect_no_error(capture.output(print(gb_place_based(a = a, b = b))))
})

#' Optimism bias upper bound for a project category
#'
#' Returns the indicative upper-bound optimism bias percentage from
#' HM Treasury Supplementary Green Book Guidance: Optimism Bias
#' (Mott MacDonald 2002, Annex A1). The upper bound is the starting
#' value for ex-ante uplift; mitigation factors reduce it as project
#' definition matures through SOC, OBC, and FBC stages.
#'
#' @param category Character. One of
#'   `"standard_buildings"`, `"non_standard_buildings"`,
#'   `"standard_civil_engineering"`,
#'   `"non_standard_civil_engineering"`,
#'   `"equipment_development"`, `"outsourcing"`.
#' @param dimension Character. `"capex"` (default) for capital
#'   expenditure uplift, or `"duration"` for works-duration uplift.
#'
#' @return Numeric scalar: the upper-bound percentage as a decimal
#'   (e.g. 0.51 for 51 percent).
#'
#' @references
#' HM Treasury (2003). Supplementary Green Book Guidance: Optimism
#' Bias.
#'
#' Mott MacDonald (2002). Review of Large Public Procurement in the
#' UK. Report commissioned by HM Treasury.
#'
#' @family optimism bias
#' @seealso [gb_apply_ob()], [gb_categories()], [gb_appraise()].
#'
#' @export
#' @examples
#' gb_optimism_bias("non_standard_buildings")
#' gb_optimism_bias("standard_civil_engineering", dimension = "duration")
gb_optimism_bias <- function(category, dimension = "capex") {
  dimension <- match.arg(dimension, c("capex", "duration"))
  tbl <- .read_optimism_bias()
  category <- match.arg(category, tbl$category)
  col <- if (dimension == "capex") "capex_upper" else "duration_upper"
  tbl[[col]][tbl$category == category]
}

#' Apply an optimism bias uplift to a cost stream
#'
#' Applies an OB uplift, optionally with a mitigation factor that
#' represents progress on risk identification and management:
#' `cost_with_ob = cost_baseline * (1 + ob_pct * (1 - mitigation))`.
#'
#' @param values Numeric vector of baseline cost values.
#' @param ob_pct Numeric scalar. Optimism bias percentage as a
#'   decimal. Pass `gb_optimism_bias(category)` to use the published
#'   upper bound.
#' @param mitigation Numeric scalar in `[0, 1]`. Fraction of the
#'   upper bound that has been mitigated through project definition
#'   and risk management. Default `0` (no mitigation; full upper
#'   bound applied).
#'
#' @return A numeric vector the same length as `values`, with the
#'   uplift applied.
#'
#' @references HM Treasury (2003). Supplementary Green Book Guidance:
#'   Optimism Bias, Annex A2 on mitigation factors.
#'
#' @family optimism bias
#' @seealso [gb_optimism_bias()], [gb_appraise()].
#'
#' @export
#' @examples
#' costs <- c(100, 50, 50)
#' ob <- gb_optimism_bias("non_standard_buildings")
#' gb_apply_ob(costs, ob)
#' gb_apply_ob(costs, ob, mitigation = 0.5)
gb_apply_ob <- function(values, ob_pct, mitigation = 0) {
  validate_numeric(values, "values")
  validate_numeric(ob_pct, "ob_pct")
  validate_numeric(mitigation, "mitigation")
  if (length(ob_pct) != 1L) cli::cli_abort("{.arg ob_pct} must be scalar.")
  if (length(mitigation) != 1L) cli::cli_abort("{.arg mitigation} must be scalar.")
  if (mitigation < 0 || mitigation > 1) {
    cli::cli_abort("{.arg mitigation} must be in [0, 1].")
  }
  values * (1 + ob_pct * (1 - mitigation))
}

#' Available optimism bias categories
#'
#' Returns the bundled OB category lookup as a data frame.
#'
#' @return A data frame with columns `category`, `description`,
#'   `capex_upper`, `duration_upper`.
#'
#' @references HM Treasury (2003). Supplementary Green Book Guidance:
#'   Optimism Bias.
#'
#' @family optimism bias
#' @seealso [gb_optimism_bias()].
#'
#' @export
#' @examples
#' gb_categories()
gb_categories <- function() {
  tbl <- .read_optimism_bias()
  tbl[, c("category", "description", "capex_upper", "duration_upper")]
}

test_that("gb_optimism_bias returns published upper bounds for capex", {
  expect_equal(gb_optimism_bias("standard_buildings"), 0.24)
  expect_equal(gb_optimism_bias("non_standard_buildings"), 0.51)
  expect_equal(gb_optimism_bias("standard_civil_engineering"), 0.44)
  expect_equal(gb_optimism_bias("non_standard_civil_engineering"), 0.66)
  expect_equal(gb_optimism_bias("equipment_development"), 2.00)
  expect_equal(gb_optimism_bias("outsourcing"), 0.41)
})

test_that("gb_optimism_bias returns published upper bounds for duration", {
  expect_equal(gb_optimism_bias("standard_buildings", "duration"), 0.04)
  expect_equal(gb_optimism_bias("non_standard_buildings", "duration"), 0.39)
  expect_equal(gb_optimism_bias("equipment_development", "duration"), 0.54)
})

test_that("gb_optimism_bias errors on unknown category", {
  expect_error(gb_optimism_bias("unknown_category"))
})

test_that("gb_optimism_bias errors on bad dimension", {
  expect_error(gb_optimism_bias("standard_buildings", dimension = "wrong"))
})

test_that("gb_apply_ob applies the upper bound when no mitigation", {
  costs <- c(100, 50)
  out <- gb_apply_ob(costs, ob_pct = 0.51)
  expect_equal(out, c(151, 75.5))
})

test_that("gb_apply_ob applies mitigation correctly", {
  costs <- 100
  out <- gb_apply_ob(costs, ob_pct = 0.50, mitigation = 0.5)
  expect_equal(out, 100 * (1 + 0.50 * 0.5))
  expect_equal(out, 125)
})

test_that("gb_apply_ob mitigation = 1 returns baseline", {
  costs <- c(100, 200)
  out <- gb_apply_ob(costs, ob_pct = 0.5, mitigation = 1)
  expect_equal(out, costs)
})

test_that("gb_apply_ob errors on out-of-range mitigation", {
  expect_error(gb_apply_ob(100, 0.5, mitigation = -0.1))
  expect_error(gb_apply_ob(100, 0.5, mitigation = 1.1))
})

test_that("gb_apply_ob errors on non-scalar ob_pct or mitigation", {
  expect_error(gb_apply_ob(100, c(0.3, 0.5)))
  expect_error(gb_apply_ob(100, 0.3, mitigation = c(0.1, 0.2)))
})

test_that("gb_categories returns all six categories", {
  cats <- gb_categories()
  expect_s3_class(cats, "data.frame")
  expect_equal(nrow(cats), 6L)
  expect_true(all(c("category", "description", "capex_upper", "duration_upper")
                  %in% names(cats)))
})

#' Optimism bias sensitivity sweep
#'
#' Recomputes an appraisal across a vector of optimism bias
#' mitigation factors. Convenience wrapper for the standard HMT
#' OB sensitivity test required at SOC, OBC, and FBC stages.
#'
#' @param costs Numeric vector of cost values.
#' @param benefits Numeric vector of benefit values.
#' @param ob Either a category name or a numeric OB percentage.
#' @param mitigations Numeric vector in `[0, 1]`. Mitigation
#'   fractions to test. Default `c(0, 0.25, 0.5, 0.75, 1.0)`.
#' @param ... Passed to `gb_appraise()`.
#'
#' @return A `gb_sensitivity_ob` object: a list with elements
#'   `mitigations`, `npv`, `bcr`, `costs_pv`.
#'
#' @family appraisal
#' @seealso [gb_appraise()], [gb_apply_ob()].
#'
#' @export
#' @examples
#' costs <- c(100, 50, 50, 0, 0, 0, 0, 0, 0, 0)
#' benefits <- c(0, 0, 0, 30, 30, 30, 30, 30, 30, 30)
#' gb_sensitivity_ob(costs, benefits, ob = "non_standard_buildings")
gb_sensitivity_ob <- function(costs, benefits,
                              ob,
                              mitigations = c(0, 0.25, 0.5, 0.75, 1.0),
                              ...) {
  validate_numeric(mitigations, "mitigations")
  if (any(mitigations < 0 | mitigations > 1)) {
    cli::cli_abort("{.arg mitigations} must all be in [0, 1].")
  }

  results <- lapply(mitigations, function(m) {
    app <- gb_appraise(costs, benefits, ob = ob, ob_mitigation = m, ...)
    data.frame(
      mitigation = m,
      npv = app$npv,
      bcr = app$bcr %||% NA_real_,
      pv_costs = app$pv_costs %||% NA_real_,
      stringsAsFactors = FALSE
    )
  })
  tbl <- do.call(rbind, results)

  out <- list(
    mitigations = mitigations,
    table = tbl,
    npv = tbl$npv,
    bcr = tbl$bcr
  )
  class(out) <- c("gb_sensitivity_ob", "list")
  out
}

#' @export
print.gb_sensitivity_ob <- function(x, ...) {
  cli::cli_h1("Optimism bias sensitivity")
  cat("\n")
  print(x$table, row.names = FALSE)
  invisible(x)
}

test_that("gb_sensitivity_ob returns a table for each mitigation", {
  costs <- c(100, 0, 0, 0, 0)
  benefits <- c(0, 30, 30, 30, 30)
  s <- gb_sensitivity_ob(costs, benefits, ob = "non_standard_buildings")
  expect_s3_class(s, "gb_sensitivity_ob")
  expect_equal(nrow(s$table), 5L)
})

test_that("gb_sensitivity_ob NPV monotonically increases with mitigation", {
  costs <- c(100, 0, 0, 0, 0)
  benefits <- c(0, 30, 30, 30, 30)
  s <- gb_sensitivity_ob(costs, benefits, ob = 0.50)
  expect_true(all(diff(s$npv) > 0))
})

test_that("gb_sensitivity_ob accepts custom mitigation grid", {
  s <- gb_sensitivity_ob(c(100, 0), c(0, 200),
                         ob = 0.50, mitigations = c(0, 0.5, 1.0))
  expect_equal(nrow(s$table), 3L)
})

test_that("gb_sensitivity_ob errors on out-of-range mitigation", {
  expect_error(gb_sensitivity_ob(c(100, 0), c(0, 200), ob = 0.5,
                                 mitigations = c(0, 1.5)))
})

test_that("gb_sensitivity_ob print method does not error", {
  s <- gb_sensitivity_ob(c(100, 0), c(0, 200), ob = 0.5)
  expect_no_error(capture.output(print(s)))
})

#' Social Time Preference Rate
#'
#' Returns the HM Treasury Green Book Social Time Preference Rate (STPR)
#' for a vector of years from a base year. The schedule is kinked: the
#' rate steps down at 30, 75, 125, 200, and 300 years. Three variants
#' are supported.
#'
#' @param years Integer vector of years from the base year (year 0 is
#'   the base year). Must be non-negative.
#' @param schedule One of `"standard"` (default, 3.5 percent baseline),
#'   `"health"` (1.5 percent baseline, used in DHSC supplementary
#'   guidance), or `"catastrophic"` (3.0 percent, for projects where
#'   catastrophic risk dominates).
#'
#' @return A numeric vector of discount rates (decimals, e.g. 0.035 for
#'   3.5 percent), one per element of `years`.
#'
#' @details
#' The STPR is composed of pure time preference plus a wealth-effect
#' adjustment for expected per-capita consumption growth. The kink
#' reflects increasing uncertainty over the constituent parameters at
#' longer horizons.
#'
#' @references
#' HM Treasury (2022). The Green Book: Central Government Guidance on
#' Appraisal and Evaluation, Annex A6.
#'
#' HM Treasury (2003). Green Book Supplementary Guidance: Discounting.
#'
#' @family discounting
#' @seealso [gb_discount_factor()], [gb_discount()], [gb_npv()].
#'
#' @export
#' @examples
#' gb_stpr(0:5)
#' gb_stpr(c(10, 30, 31, 75, 76))
#' gb_stpr(c(10, 30, 31, 75, 76), schedule = "health")
gb_stpr <- function(years, schedule = "standard") {
  schedule <- match.arg(schedule, c("standard", "health", "catastrophic"))
  validate_year(years, arg = "years", min_year = 0L)

  schedule_table <- .read_stpr()

  out <- numeric(length(years))
  for (i in seq_along(years)) {
    y <- years[i]
    row_idx <- which(y >= schedule_table$year_from & y <= schedule_table$year_to)
    if (length(row_idx) == 0L) {
      cli::cli_abort(
        "Year {.val {y}} is outside the supported STPR schedule (max: {.val {max(schedule_table$year_to)}})."
      )
    }
    out[i] <- schedule_table[[schedule]][row_idx[[1]]]
  }
  out
}

test_that("gb_stpr returns headline 3.5 percent for first band", {
  expect_equal(gb_stpr(0), 0.035)
  expect_equal(gb_stpr(1), 0.035)
  expect_equal(gb_stpr(15), 0.035)
  expect_equal(gb_stpr(30), 0.035)
})

test_that("gb_stpr steps down at year 31", {
  expect_equal(gb_stpr(31), 0.030)
  expect_equal(gb_stpr(50), 0.030)
  expect_equal(gb_stpr(75), 0.030)
})

test_that("gb_stpr steps down across all bands", {
  expect_equal(gb_stpr(76), 0.025)
  expect_equal(gb_stpr(125), 0.025)
  expect_equal(gb_stpr(126), 0.020)
  expect_equal(gb_stpr(200), 0.020)
  expect_equal(gb_stpr(201), 0.015)
  expect_equal(gb_stpr(300), 0.015)
  expect_equal(gb_stpr(301), 0.010)
  expect_equal(gb_stpr(1000), 0.010)
})

test_that("gb_stpr is vectorised", {
  out <- gb_stpr(c(10, 30, 31, 75, 76))
  expect_equal(out, c(0.035, 0.035, 0.030, 0.030, 0.025))
  expect_length(out, 5L)
})

test_that("gb_stpr supports the health schedule", {
  expect_equal(gb_stpr(0, "health"), 0.015)
  expect_equal(gb_stpr(31, "health"), 0.0129)
  expect_equal(gb_stpr(76, "health"), 0.0107)
})

test_that("gb_stpr supports the catastrophic schedule", {
  expect_equal(gb_stpr(0, "catastrophic"), 0.030)
  expect_equal(gb_stpr(31, "catastrophic"), 0.0257)
  expect_equal(gb_stpr(76, "catastrophic"), 0.0214)
})

test_that("gb_stpr errors on invalid input", {
  expect_error(gb_stpr(-1), "must be >= 0")
  expect_error(gb_stpr(1.5), "integer-valued")
  expect_error(gb_stpr("abc"), "must be a numeric")
  expect_error(gb_stpr(NA_real_), "NA")
  expect_error(gb_stpr(0, schedule = "wrong"))
})

test_that("gb_stpr errors above the last band", {
  expect_error(gb_stpr(1001), "outside the supported")
})

#' Wellbeing valuation in GBP (WELLBY)
#'
#' Monetises a life-satisfaction change as Well-being Adjusted
#' Life Years (WELLBYs) per HMT Wellbeing Guidance for Appraisal
#' (July 2021). One WELLBY equals a one-point change in life
#' satisfaction on a 0 to 10 scale, for one person, for one year.
#' The central published unit value is GBP 13,000 in 2019 prices,
#' with a low-high sensitivity of GBP 10,000 to GBP 16,000.
#'
#' @param life_satisfaction_change Numeric scalar or vector. Change
#'   in life satisfaction on the 0 to 10 scale (signed; can be
#'   positive or negative).
#' @param persons Number of people experiencing the change.
#' @param years Duration in years. Default `1`.
#' @param base_year Optional integer base year to express the
#'   monetary value in. If `NULL` (default), the published 2019
#'   price is returned. Otherwise the value is uplifted via
#'   `gb_deflator()` to `base_year` prices.
#' @param scenario One of `"low"`, `"central"` (default), `"high"`.
#'
#' @return A numeric scalar or vector: the WELLBY value in GBP at
#'   the requested base year.
#'
#' @references HM Treasury (2021). Wellbeing Guidance for Appraisal:
#'   Supplementary Green Book Guidance.
#'
#' @family valuation
#' @seealso [gb_vpf()], [gb_qaly()].
#'
#' @export
#' @examples
#' # 1-point lift in life satisfaction for 100 people for 5 years
#' gb_wellby(1, persons = 100, years = 5)
#' gb_wellby(1, persons = 100, years = 5, scenario = "low")
#' # Express in 2024 prices
#' gb_wellby(1, persons = 100, years = 5, base_year = 2024)
gb_wellby <- function(life_satisfaction_change, persons, years = 1,
                      base_year = NULL, scenario = "central") {
  validate_numeric(life_satisfaction_change, "life_satisfaction_change")
  validate_numeric(persons, "persons", require_positive = FALSE)
  validate_numeric(years, "years")
  scenario <- match.arg(scenario, c("low", "central", "high"))

  tbl <- .read_wellby()
  unit_2019 <- tbl$value_gbp[tbl$scenario == scenario]
  pub_base <- tbl$base_year[tbl$scenario == scenario]

  if (!is.null(base_year) && base_year != pub_base) {
    unit <- unit_2019 * gb_deflator(pub_base, base_year)
  } else {
    unit <- unit_2019
  }

  life_satisfaction_change * persons * years * unit
}

#' Value of Preventing a Fatality
#'
#' Returns the DfT TAG-published Value of Preventing a Fatality
#' (VPF) in real terms for a given year. The published 2024 value
#' is GBP 2.153 million; the historical 2018 value (DfT TAG) is
#' GBP 1.958 million. Years between bundled publication dates are
#' filled by an annual real uplift of approximately 2 percent
#' (proxy for real GDP per head growth).
#'
#' @param year Integer scalar. The year in which the value is
#'   expressed. Default `2024` (the most recent DfT-published
#'   value).
#' @param series Character. `"central"` (default). Reserved for
#'   future expansion (DfT cancer / aversion multipliers).
#'
#' @return Numeric scalar: the VPF in GBP at year `year` prices.
#'
#' @references Department for Transport. Transport Analysis
#'   Guidance (TAG) data book.
#'
#' @family valuation
#' @seealso [gb_wellby()], [gb_qaly()].
#'
#' @export
#' @examples
#' gb_vpf()
#' gb_vpf(2018)
gb_vpf <- function(year = 2024, series = "central") {
  series <- match.arg(series, c("central"))
  validate_year(year, "year", min_year = 1900L)
  if (length(year) != 1L) cli::cli_abort("{.arg year} must be scalar.")

  tbl <- .read_vpf()
  if (!(year %in% tbl$year)) {
    cli::cli_abort(
      "Year {.val {year}} outside bundled VPF range ({min(tbl$year)} to {max(tbl$year)})."
    )
  }
  tbl$value_gbp[tbl$year == year]
}

#' Value of a Quality-Adjusted Life Year
#'
#' Returns the QALY value in GBP at the published base year. The
#' DHSC supplementary Green Book guidance specifies GBP 70,000 per
#' QALY in 2024 prices for cross-government appraisal. NICE Health
#' Technology Assessment uses lower thresholds (GBP 20,000 to
#' GBP 30,000 per QALY).
#'
#' @param qalys Numeric scalar or vector. Quality-Adjusted Life
#'   Years gained or lost.
#' @param scenario One of `"dhsc"` (default, GBP 70k 2024 prices,
#'   cross-government), `"nice_lower"` (GBP 20k), `"nice_upper"`
#'   (GBP 30k).
#' @param base_year Optional integer base year. If `NULL`, the
#'   published base year for the chosen scenario is used.
#'
#' @return A numeric scalar or vector: monetised value in GBP.
#'
#' @references DHSC Supplementary Green Book Guidance on health
#'   appraisal. NICE methods guides for technology appraisal.
#'
#' @family valuation
#' @seealso [gb_wellby()], [gb_vpf()].
#'
#' @export
#' @examples
#' gb_qaly(1)
#' gb_qaly(1, scenario = "nice_upper")
gb_qaly <- function(qalys, scenario = "dhsc", base_year = NULL) {
  validate_numeric(qalys, "qalys")
  scenario <- match.arg(scenario, c("dhsc", "nice_lower", "nice_upper"))

  tbl <- .read_qaly()
  unit <- tbl$value_gbp[tbl$scenario == scenario]
  pub_base <- tbl$base_year[tbl$scenario == scenario]

  if (!is.null(base_year) && base_year != pub_base) {
    unit <- unit * gb_deflator(pub_base, base_year)
  }

  qalys * unit
}

test_that("gb_wellby central value matches HMT 2019 figure", {
  out <- gb_wellby(1, persons = 1, years = 1)
  expect_equal(out, 13000)
})

test_that("gb_wellby low and high scenarios match HMT range", {
  expect_equal(gb_wellby(1, persons = 1, years = 1, scenario = "low"), 10000)
  expect_equal(gb_wellby(1, persons = 1, years = 1, scenario = "high"), 16000)
})

test_that("gb_wellby scales linearly", {
  expect_equal(gb_wellby(0.5, 200, 3), 0.5 * 200 * 3 * 13000)
})

test_that("gb_wellby uplifts to other base years via deflator", {
  v_2019 <- gb_wellby(1, 1, 1, base_year = 2019)
  v_2024 <- gb_wellby(1, 1, 1, base_year = 2024)
  factor <- gb_deflator(2019, 2024)
  expect_equal(v_2024, v_2019 * factor, tolerance = 1e-6)
})

test_that("gb_wellby errors on bad scenario", {
  expect_error(gb_wellby(1, 1, scenario = "wrong"))
})

test_that("gb_vpf 2023 anchor matches DfT TAG A4.1.1 WTP", {
  expect_equal(gb_vpf(2023), 2474341)
})

test_that("gb_vpf default (2024) is uplifted at 2 percent real", {
  expect_equal(gb_vpf(), round(2474341 * 1.02))
})

test_that("gb_vpf 2018 is downlifted from 2023 anchor", {
  expect_equal(gb_vpf(2018), round(2474341 / 1.02 ^ 5))
})

test_that("gb_vpf monotonically increases over time", {
  out <- vapply(2018:2030, gb_vpf, numeric(1))
  expect_true(all(diff(out) > 0))
})

test_that("gb_vpf errors outside bundled range", {
  expect_error(gb_vpf(2010))
  expect_error(gb_vpf(2050))
})

test_that("gb_vpf errors on non-scalar year", {
  expect_error(gb_vpf(c(2020, 2024)))
})

test_that("gb_qaly default is DHSC GBP 70k per QALY", {
  expect_equal(gb_qaly(1), 70000)
})

test_that("gb_qaly NICE thresholds match published values", {
  expect_equal(gb_qaly(1, scenario = "nice_lower"), 20000)
  expect_equal(gb_qaly(1, scenario = "nice_upper"), 30000)
})

test_that("gb_qaly scales linearly", {
  expect_equal(gb_qaly(2.5), 2.5 * 70000)
  expect_equal(gb_qaly(c(1, 2, 3)), c(70000, 140000, 210000))
})

test_that("gb_qaly rebases via deflator", {
  v_2024 <- gb_qaly(1, base_year = 2024)
  v_2020 <- gb_qaly(1, base_year = 2020)
  factor <- gb_deflator(2024, 2020)
  expect_equal(v_2020, v_2024 * factor, tolerance = 1e-6)
})

test_that("gb_qaly errors on bad scenario", {
  expect_error(gb_qaly(1, scenario = "wrong"))
})

#' Discount factor under the kinked STPR
#'
#' Returns the present-value discount factor for each year, applying
#' the kinked Social Time Preference Rate schedule annually.
#'
#' @param years Integer vector of years from the base year. Must be
#'   `>= base_year`.
#' @param schedule One of `"standard"`, `"health"`, `"catastrophic"`.
#' @param base_year Integer base year offset. Default `0`.
#'
#' @return A numeric vector of discount factors. `1` for `years == base_year`.
#'
#' @details
#' The discount factor at year `t` is computed as the reciprocal of
#' the cumulative product of `(1 + r_k)` for periods `k = 1, ..., t`,
#' where `r_k` is the STPR for year `k`. This handles the kinked
#' schedule correctly across band transitions (e.g. year 30 to year
#' 31). Within a single band, the closed-form annuity factor
#' `(1 - (1 + r)^(-n)) / r` agrees with `sum(gb_discount_factor(1:n))`
#' to machine precision.
#'
#' @references
#' HM Treasury (2022). The Green Book, Annex A6.
#'
#' HM Treasury (2003). Green Book Supplementary Guidance: Discounting.
#'
#' @family discounting
#' @seealso [gb_stpr()], [gb_npv()], [gb_eanc()].
#'
#' @export
#' @examples
#' gb_discount_factor(0:5)
#' gb_discount_factor(c(0, 30, 31, 75, 76))
gb_discount_factor <- function(years, schedule = "standard", base_year = 0L) {
  validate_year(years, arg = "years", min_year = -.Machine$integer.max)
  validate_year(base_year, arg = "base_year", min_year = -.Machine$integer.max)
  if (length(base_year) != 1L) {
    cli::cli_abort("{.arg base_year} must be scalar.")
  }
  if (any(years < base_year)) {
    cli::cli_abort("All {.arg years} must be {.code >=} {.arg base_year} ({.val {base_year}}).")
  }

  rel <- as.integer(years - base_year)
  if (length(rel) == 0L) return(numeric(0))

  max_y <- max(rel)
  if (max_y == 0L) return(rep_len(1, length(rel)))

  period_rates <- gb_stpr(seq_len(max_y), schedule = schedule)
  cumulative <- cumprod(1 + period_rates)

  out <- numeric(length(rel))
  out[rel == 0L] <- 1
  nonzero <- rel > 0L
  out[nonzero] <- 1 / cumulative[rel[nonzero]]
  out
}

#' Apply discount factors to a stream
#'
#' @param values Numeric vector of cashflow values (real terms, base
#'   year fixed).
#' @param years Integer vector of years from the base year. Defaults
#'   to `0, 1, 2, ...`.
#' @param schedule One of `"standard"`, `"health"`, `"catastrophic"`.
#'
#' @return A numeric vector of discounted (present-value) cashflows.
#' @family discounting
#' @seealso [gb_discount_factor()], [gb_npv()].
#' @references HM Treasury (2022). The Green Book, Annex A6.
#' @export
#' @examples
#' gb_discount(c(0, 100, 100, 100))
gb_discount <- function(values, years = seq_along(values) - 1L, schedule = "standard") {
  validate_numeric(values, "values")
  validate_year(years, "years", min_year = -.Machine$integer.max)
  if (length(values) != length(years)) {
    cli::cli_abort("{.arg values} and {.arg years} must have equal length.")
  }
  values * gb_discount_factor(years, schedule = schedule)
}

#' Net present value
#'
#' Computes the net present value of a cashflow stream under the
#' Green Book STPR. Returns a `gb_appraisal` object carrying the NPV,
#' the input cashflow, the discount factors, the schedule used, and
#' methodology vintage.
#'
#' @param cashflow Numeric vector of net cashflows in real terms (one
#'   value per year).
#' @param years Integer vector of years matching `cashflow`. Defaults
#'   to `0, 1, ..., length(cashflow) - 1`.
#' @param schedule One of `"standard"`, `"health"`, `"catastrophic"`.
#' @param base_year Optional integer recording the price base year for
#'   the cashflow (e.g. `2024`). Stored on the returned object.
#' @param vintage Character. Methodology vintage label. Defaults to
#'   `"2022"`.
#'
#' @return A `gb_appraisal` object: a list with class
#'   `c("gb_appraisal", "list")` and elements
#'   `npv`, `cashflow`, `years`, `pv`, `schedule`, `base_year`, `vintage`.
#'
#' @references
#' HM Treasury (2022). The Green Book, chapter on appraisal and the
#' Annex on discounting.
#'
#' @family discounting
#' @seealso [gb_appraise()], [gb_eanc()], [gb_dist_weighted_npv()],
#'   [gb_carbon_npv()].
#'
#' @export
#' @examples
#' costs    <- c(100, 50, 50, 0, 0, 0, 0, 0, 0, 0)
#' benefits <- c(0, 0, 0, 30, 30, 30, 30, 30, 30, 30)
#' gb_npv(benefits - costs)
gb_npv <- function(cashflow,
                   years = NULL,
                   schedule = "standard",
                   base_year = NULL,
                   vintage = "2022") {
  validate_numeric(cashflow, "cashflow")
  schedule <- match.arg(schedule, c("standard", "health", "catastrophic"))
  if (is.null(years)) years <- seq_along(cashflow) - 1L
  pv <- gb_discount(cashflow, years, schedule = schedule)

  out <- list(
    npv = sum(pv),
    cashflow = cashflow,
    years = years,
    pv = pv,
    schedule = schedule,
    base_year = base_year,
    vintage = vintage
  )
  class(out) <- c("gb_appraisal", "list")
  out
}

#' Equivalent annual net cost (or benefit)
#'
#' Converts a present value to an annualised equivalent over a fixed
#' horizon under the STPR. Used to compare options of different
#' durations.
#'
#' @param npv Either a numeric scalar NPV, or a `gb_appraisal` object.
#' @param years Project horizon in years. If `npv` is a `gb_appraisal`
#'   and `years` is missing, `length(npv$cashflow) - 1L` is used.
#' @param schedule One of `"standard"`, `"health"`, `"catastrophic"`.
#'
#' @return A numeric scalar: the equivalent annual amount in real
#'   terms, base year aligned with the input.
#'
#' @references HM Treasury (2022). The Green Book, Annex A on
#'   appraisal: equivalent annual cost.
#'
#' @family discounting
#' @seealso [gb_npv()], [gb_appraise()].
#'
#' @export
#' @examples
#' app <- gb_npv(c(-100, 30, 30, 30, 30, 30))
#' gb_eanc(app)
gb_eanc <- function(npv, years, schedule = "standard") {
  if (inherits(npv, "gb_appraisal")) {
    npv_val <- npv$npv
    if (missing(years) || is.null(years)) {
      years <- length(npv$cashflow) - 1L
    }
    schedule <- npv$schedule %||% schedule
  } else if (is.numeric(npv) && length(npv) == 1L) {
    npv_val <- npv
  } else {
    cli::cli_abort("{.arg npv} must be a numeric scalar or a {.cls gb_appraisal}.")
  }
  validate_year(years, "years", min_year = 1L)
  if (length(years) != 1L) {
    cli::cli_abort("{.arg years} must be scalar.")
  }

  factors <- gb_discount_factor(seq_len(years), schedule = schedule)
  npv_val / sum(factors)
}

test_that("print.gb_appraisal returns the input invisibly", {
  app <- gb_npv(c(-100, 50, 50, 50))
  capture.output(out <- print(app))
  expect_identical(out, app)
})

test_that("print.gb_appraisal does not error", {
  app <- gb_npv(c(-100, 50, 50, 50), base_year = 2024)
  expect_no_error(capture.output(print(app)))
})

test_that("summary.gb_appraisal returns the input invisibly", {
  app <- gb_npv(c(-100, 50, 50, 50))
  capture.output(out <- summary(app))
  expect_identical(out, app)
})

test_that("summary.gb_appraisal prints provenance", {
  app <- gb_npv(c(-100, 50, 50, 50), base_year = 2024)
  out <- capture.output(summary(app))
  expect_true(any(grepl("NPV", out)))
  expect_true(any(grepl("standard", out)))
  expect_true(any(grepl("Vintage", out)))
})

test_that("format.gb_appraisal returns character", {
  app <- gb_npv(c(-100, 50, 50, 50))
  s <- format(app)
  expect_type(s, "character")
  expect_length(s, 1L)
  expect_match(s, "gb_appraisal")
  expect_match(s, "NPV")
})

# Cross-validation tests pinning the kinked-schedule logic against
# closed-form benchmarks. Useful as level-3/4 invariant tests for
# academic and government-economics scrutiny.

test_that("annuity factor at 3.5 percent for 30 years matches closed form", {
  factors <- gb_discount_factor(seq_len(30))
  annuity <- sum(factors)
  closed_form <- (1 - 1.035^-30) / 0.035
  expect_equal(annuity, closed_form, tolerance = 1e-9)
})

test_that("annuity factor for 5 years matches closed form", {
  factors <- gb_discount_factor(seq_len(5))
  annuity <- sum(factors)
  closed_form <- (1 - 1.035^-5) / 0.035
  expect_equal(annuity, closed_form, tolerance = 1e-9)
})

test_that("100-year flat GBP 1 annuity matches kinked-schedule benchmark", {
  rates <- c(rep(0.035, 30), rep(0.030, 45), rep(0.025, 25))
  factors_manual <- cumprod(1 + rates)
  benchmark <- sum(1 / factors_manual)
  package <- sum(gb_discount_factor(seq_len(100)))
  expect_equal(package, benchmark, tolerance = 1e-9)
})

test_that("health and catastrophic rates are constant ratios of standard within first band", {
  # Within the first band (years 1-30), all variants have constant rates.
  health_ratio <- gb_stpr(15, "health") / gb_stpr(15, "standard")
  catastrophic_ratio <- gb_stpr(15, "catastrophic") / gb_stpr(15, "standard")
  expect_equal(health_ratio, 0.015 / 0.035, tolerance = 1e-9)
  expect_equal(catastrophic_ratio, 0.030 / 0.035, tolerance = 1e-9)
})

test_that("gb_npv worked example: 5-year annuity matches manual calculation", {
  # Standard appraisal pattern: capex GBP 100 in year 0, benefits
  # GBP 30 per year in years 1-5
  net <- c(-100, rep(30, 5))
  app <- gb_npv(net)
  manual <- -100 + sum(30 / 1.035 ^ (1:5))
  expect_equal(app$npv, manual, tolerance = 1e-9)
})

test_that("gb_appraise: 30-year project with OB and METB matches manual", {
  costs <- c(100, rep(0, 29))
  benefits <- c(0, rep(10, 29))
  app <- gb_appraise(costs, benefits, ob = 0.20, ob_mitigation = 0.5,
                     metb = TRUE, metb_rate = 0.20)

  # Manual: cost uplifted by OB then METB:
  # capex_adj = 100 * (1 + 0.20 * 0.5) = 110
  # then * (1 + 0.20) = 132
  expect_equal(app$costs[1], 132, tolerance = 1e-9)

  # PV of benefits: 10 * sum(1.035^-t, t=1..29)
  benefits_pv_manual <- 10 * sum(1.035 ^ -(1:29))
  expect_equal(app$pv_benefits, benefits_pv_manual, tolerance = 1e-9)

  # NPV: PV benefits - PV costs (capex at year 0 has DF = 1)
  expect_equal(app$npv, benefits_pv_manual - 132, tolerance = 1e-9)
})

test_that("WELLBY scaling: persons and years are linear and commute", {
  base <- gb_wellby(1, persons = 1, years = 1)
  expect_equal(gb_wellby(1, 100, 5), 100 * 5 * base, tolerance = 1e-9)
  expect_equal(gb_wellby(1, 5, 100), 5 * 100 * base, tolerance = 1e-9)
})

test_that("gb_carbon_npv equals manually-discounted emissions value", {
  emissions <- rep(100, 5)
  years <- 2024:2028
  app <- gb_carbon_npv(emissions, years, base_year = 2024)

  values <- gb_carbon_value(years, base_year = 2024)
  cashflow <- -emissions * values  # cost convention
  rel_years <- years - 2024
  manual_pv <- sum(cashflow * gb_discount_factor(rel_years))
  expect_equal(app$npv, manual_pv, tolerance = 1e-9)
})

test_that("citation() returns greenbook-aware bibentry", {
  cit <- utils::citation("greenbook")
  expect_s3_class(cit, "citation")
  txt <- format(cit)
  expect_true(any(grepl("greenbook", txt, ignore.case = TRUE)))
  expect_true(any(grepl("Coverdale", txt)))
})

test_that("gb_discount_factor at year 0 is 1", {
  expect_equal(gb_discount_factor(0), 1)
})

test_that("gb_discount_factor at year 1 is 1/1.035", {
  expect_equal(gb_discount_factor(1), 1 / 1.035, tolerance = 1e-9)
})

test_that("gb_discount_factor compounds annually within a band", {
  expect_equal(gb_discount_factor(5), 1 / 1.035^5, tolerance = 1e-9)
  expect_equal(gb_discount_factor(30), 1 / 1.035^30, tolerance = 1e-9)
})

test_that("gb_discount_factor handles the kink at year 31", {
  df_30 <- gb_discount_factor(30)
  df_31 <- gb_discount_factor(31)
  expect_equal(df_31, df_30 / 1.030, tolerance = 1e-9)
})

test_that("gb_discount_factor handles two kink transitions", {
  df_75 <- 1 / (1.035^30 * 1.030^45)
  expect_equal(gb_discount_factor(75), df_75, tolerance = 1e-9)
  df_76 <- df_75 / 1.025
  expect_equal(gb_discount_factor(76), df_76, tolerance = 1e-9)
})

test_that("gb_discount_factor is vectorised", {
  out <- gb_discount_factor(c(0, 1, 5, 30, 31))
  expect_length(out, 5L)
  expect_equal(out[1], 1)
  expect_equal(out[2], 1 / 1.035, tolerance = 1e-9)
})

test_that("gb_discount_factor is monotonically decreasing", {
  out <- gb_discount_factor(0:50)
  expect_true(all(diff(out) < 0))
})

test_that("gb_discount_factor with health schedule is higher than standard", {
  expect_gt(
    gb_discount_factor(30, schedule = "health"),
    gb_discount_factor(30, schedule = "standard")
  )
})

test_that("gb_discount_factor errors on years below base_year", {
  expect_error(gb_discount_factor(-1, base_year = 0))
  expect_error(gb_discount_factor(2, base_year = 5))
})

test_that("gb_discount applies factors element-wise", {
  values <- c(100, 100, 100, 100)
  expected <- values * gb_discount_factor(0:3)
  expect_equal(gb_discount(values), expected)
})

test_that("gb_discount errors on length mismatch", {
  expect_error(gb_discount(c(1, 2, 3), years = c(0, 1)), "equal length")
})

test_that("gb_npv returns a gb_appraisal object", {
  app <- gb_npv(c(-100, 50, 50, 50))
  expect_s3_class(app, "gb_appraisal")
  expect_named(app, c("npv", "cashflow", "years", "pv", "schedule",
                      "base_year", "vintage"))
})

test_that("gb_npv is consistent with manual discounting", {
  cf <- c(-100, 50, 50, 50)
  app <- gb_npv(cf)
  manual <- sum(cf * gb_discount_factor(0:3))
  expect_equal(app$npv, manual, tolerance = 1e-9)
})

test_that("gb_npv stores schedule and vintage", {
  app <- gb_npv(c(-100, 50), schedule = "health", vintage = "2022")
  expect_equal(app$schedule, "health")
  expect_equal(app$vintage, "2022")
})

test_that("gb_npv stores base year when supplied", {
  app <- gb_npv(c(-100, 50), base_year = 2024)
  expect_equal(app$base_year, 2024)
})

test_that("gb_npv known-value: GBP 100 capex, 5 years of GBP 30 benefit", {
  cf <- c(-100, 30, 30, 30, 30, 30)
  app <- gb_npv(cf)
  manual <- -100 + sum(30 / 1.035^(1:5))
  expect_equal(app$npv, manual, tolerance = 1e-9)
})

test_that("gb_eanc is consistent with NPV / annuity factor", {
  app <- gb_npv(c(-100, 30, 30, 30, 30, 30))
  factors <- gb_discount_factor(seq_len(5))
  expect_equal(gb_eanc(app, years = 5), app$npv / sum(factors), tolerance = 1e-9)
})

test_that("gb_eanc accepts numeric NPV scalar", {
  factors <- gb_discount_factor(seq_len(10))
  expect_equal(gb_eanc(50, years = 10), 50 / sum(factors), tolerance = 1e-9)
})

test_that("gb_eanc derives years from gb_appraisal length", {
  app <- gb_npv(c(-100, 30, 30, 30, 30, 30))
  expect_equal(gb_eanc(app), gb_eanc(app, years = 5), tolerance = 1e-12)
})

test_that("gb_eanc errors on bad inputs", {
  expect_error(gb_eanc("abc", years = 5), "numeric scalar or")
  expect_error(gb_eanc(50, years = -1), "must be >= 1")
  expect_error(gb_eanc(50, years = c(1, 2)), "must be scalar")
})

test_that("gb_npv errors on bad input", {
  expect_error(gb_npv("abc"), "must be a numeric")
  expect_error(gb_npv(c(1, NA)), "NA")
})

#' Distributional weight for a recipient income
#'
#' Returns the Green Book distributional weight applied to net
#' benefits accruing to a recipient at income `income`, relative to
#' a reference income (median by default), under iso-elastic utility:
#'
#' \deqn{w_i = (y_{ref} / y_i) ^ \eta}
#'
#' Default `eta = 1.3` per HM Treasury Green Book Annex A3. Higher
#' eta places greater weight on lower-income recipients; sensitivity
#' tests at `eta = 0.8` and `eta = 2.0` are conventional.
#'
#' @param income Numeric vector of recipient incomes (positive).
#'   Equivalised household disposable income is the conventional
#'   measure.
#' @param eta Numeric scalar. Income elasticity of marginal utility
#'   of income. Default `1.3`.
#' @param reference Either `"median"` (default; use median of
#'   `income` or `income_data` if supplied) or a numeric scalar
#'   (use that income as reference).
#' @param income_data Optional numeric vector. Reference income
#'   distribution for computing the median. Use, e.g., the ONS
#'   household disposable income distribution. If `NULL` (default),
#'   the median of `income` is used.
#'
#' @return A numeric vector of weights, same length as `income`.
#'
#' @references
#' HM Treasury (2022). The Green Book, Annex A3 on distributional
#' analysis.
#'
#' Acland, D. and Greenberg, D.H. (2024). The Elasticity of Marginal
#' Utility of Income for Distributional Weighting and Social
#' Discounting: A Meta-Analysis. Journal of Benefit-Cost Analysis.
#'
#' @family distributional
#' @seealso [gb_dist_weighted_npv()].
#'
#' @export
#' @examples
#' # Weights across deciles of a stylised income distribution
#' income_deciles <- seq(10000, 100000, length.out = 10)
#' gb_dist_weight(income_deciles)
gb_dist_weight <- function(income, eta = 1.3, reference = "median",
                           income_data = NULL) {
  validate_numeric(income, "income", require_positive = TRUE)
  validate_numeric(eta, "eta")
  if (length(eta) != 1L) cli::cli_abort("{.arg eta} must be scalar.")

  if (is.character(reference) && length(reference) == 1L && reference == "median") {
    if (!is.null(income_data)) {
      validate_numeric(income_data, "income_data", require_positive = TRUE)
      ref_income <- stats::median(income_data)
    } else {
      ref_income <- stats::median(income)
    }
  } else if (is.numeric(reference) && length(reference) == 1L) {
    if (reference <= 0) cli::cli_abort("{.arg reference} must be > 0.")
    ref_income <- reference
  } else {
    cli::cli_abort("{.arg reference} must be {.val median} or a positive numeric scalar.")
  }

  (ref_income / income) ^ eta
}

#' Distributionally-weighted net present value
#'
#' Applies recipient-income distributional weights to a cashflow
#' before discounting under the STPR.
#'
#' @param cashflow Numeric vector of net cashflows (per period).
#' @param recipient_income Numeric vector. The income of the
#'   recipient (or representative recipient cell) in each period.
#'   Must have the same length as `cashflow`.
#' @param eta Numeric scalar. Default `1.3`.
#' @param schedule One of `"standard"`, `"health"`, `"catastrophic"`.
#' @param reference Reference income strategy passed to
#'   `gb_dist_weight()`.
#' @param income_data Optional reference income distribution.
#' @param vintage Methodology vintage label. Default `"2022"`.
#' @param base_year Optional integer base year stored on the result.
#'
#' @return A `gb_appraisal` object with extra fields `weights`,
#'   `eta`, and `unweighted_npv`.
#'
#' @references HM Treasury (2022). The Green Book, Annex A3 on
#'   distributional analysis.
#'
#' @family distributional
#' @seealso [gb_dist_weight()], [gb_npv()].
#'
#' @export
#' @examples
#' # 5-year benefit stream of GBP 30 going to a low-decile recipient
#' gb_dist_weighted_npv(
#'   cashflow = rep(30, 5),
#'   recipient_income = rep(15000, 5),
#'   income_data = seq(10000, 100000, length.out = 10)
#' )
gb_dist_weighted_npv <- function(cashflow, recipient_income, eta = 1.3,
                                 schedule = "standard",
                                 reference = "median",
                                 income_data = NULL,
                                 vintage = "2022",
                                 base_year = NULL) {
  validate_numeric(cashflow, "cashflow")
  if (length(cashflow) != length(recipient_income)) {
    cli::cli_abort("{.arg cashflow} and {.arg recipient_income} must have equal length.")
  }

  weights <- gb_dist_weight(recipient_income, eta = eta,
                            reference = reference,
                            income_data = income_data)
  weighted_cashflow <- cashflow * weights

  unweighted <- gb_npv(cashflow, schedule = schedule,
                      base_year = base_year, vintage = vintage)
  weighted <- gb_npv(weighted_cashflow, schedule = schedule,
                    base_year = base_year, vintage = vintage)

  weighted$weights <- weights
  weighted$eta <- eta
  weighted$unweighted_npv <- unweighted$npv
  weighted
}

test_that("gb_dist_weight returns 1 at the reference income", {
  income <- c(20000, 50000, 80000)
  w <- gb_dist_weight(income, reference = 50000)
  expect_equal(w[2], 1, tolerance = 1e-12)
})

test_that("gb_dist_weight is > 1 below reference, < 1 above", {
  income <- c(20000, 50000, 80000)
  w <- gb_dist_weight(income, reference = 50000)
  expect_gt(w[1], 1)
  expect_lt(w[3], 1)
})

test_that("gb_dist_weight is monotonically decreasing in income", {
  income <- seq(10000, 100000, length.out = 10)
  w <- gb_dist_weight(income)
  expect_true(all(diff(w) < 0))
})

test_that("gb_dist_weight: higher eta amplifies the weights", {
  income <- c(20000, 80000)
  w_low <- gb_dist_weight(income, eta = 0.8, reference = 50000)
  w_high <- gb_dist_weight(income, eta = 2.0, reference = 50000)
  # ratio of low to high decile widens with eta
  expect_gt(w_high[1] / w_high[2], w_low[1] / w_low[2])
})

test_that("gb_dist_weight default eta is 1.3", {
  w_default <- gb_dist_weight(c(20000, 80000), reference = 50000)
  w_explicit <- gb_dist_weight(c(20000, 80000), eta = 1.3, reference = 50000)
  expect_equal(w_default, w_explicit)
})

test_that("gb_dist_weight uses median of income_data when supplied", {
  income <- c(15000)
  decile_data <- seq(10000, 100000, length.out = 10)
  ref <- stats::median(decile_data)
  w <- gb_dist_weight(income, income_data = decile_data)
  expect_equal(w, (ref / 15000) ^ 1.3, tolerance = 1e-9)
})

test_that("gb_dist_weight uses median of input when no reference", {
  income <- c(20000, 50000, 80000)
  w <- gb_dist_weight(income)
  ref <- stats::median(income)
  expect_equal(w, (ref / income) ^ 1.3, tolerance = 1e-9)
})

test_that("gb_dist_weight errors on non-positive income", {
  expect_error(gb_dist_weight(c(0, 50000)))
  expect_error(gb_dist_weight(c(-100, 50000)))
})

test_that("gb_dist_weight errors on bad reference", {
  expect_error(gb_dist_weight(50000, reference = "wrong"))
  expect_error(gb_dist_weight(50000, reference = -100))
})

test_that("gb_dist_weighted_npv returns gb_appraisal with extras", {
  app <- gb_dist_weighted_npv(rep(30, 5),
                              recipient_income = rep(20000, 5),
                              income_data = seq(10000, 100000, length.out = 10))
  expect_s3_class(app, "gb_appraisal")
  expect_true(!is.null(app$weights))
  expect_true(!is.null(app$eta))
  expect_true(!is.null(app$unweighted_npv))
})

test_that("gb_dist_weighted_npv: low income recipient gets higher weighted NPV", {
  unweighted <- gb_npv(rep(30, 5))
  decile_data <- seq(10000, 100000, length.out = 10)

  low_income <- gb_dist_weighted_npv(rep(30, 5),
                                     recipient_income = rep(15000, 5),
                                     income_data = decile_data)
  high_income <- gb_dist_weighted_npv(rep(30, 5),
                                      recipient_income = rep(80000, 5),
                                      income_data = decile_data)
  expect_gt(low_income$npv, unweighted$npv)
  expect_lt(high_income$npv, unweighted$npv)
})

test_that("gb_dist_weighted_npv errors on length mismatch", {
  expect_error(
    gb_dist_weighted_npv(c(10, 20, 30), recipient_income = c(20000, 30000))
  )
})

# Internal helpers (not exported)

`%||%` <- function(a, b) if (is.null(a)) b else a

# Path to bundled CSV in inst/extdata
.gb_extdata <- function(file) {
  path <- system.file("extdata", file, package = "greenbook")
  if (!nzchar(path)) {
    cli::cli_abort("Bundled data file {.val {file}} not found in greenbook extdata.")
  }
  path
}

# Lazy readers for bundled tables
.read_stpr <- function() {
  utils::read.csv(.gb_extdata("stpr_schedule.csv"))
}

.read_deflator <- function() {
  utils::read.csv(.gb_extdata("gdp_deflator.csv"))
}

.read_data_versions <- function() {
  utils::read.csv(.gb_extdata("data_versions.csv"))
}

.read_optimism_bias <- function() {
  utils::read.csv(.gb_extdata("optimism_bias.csv"), stringsAsFactors = FALSE)
}

.read_metb <- function() {
  utils::read.csv(.gb_extdata("metb.csv"), stringsAsFactors = FALSE)
}

.read_wellby <- function() {
  utils::read.csv(.gb_extdata("wellby.csv"), stringsAsFactors = FALSE)
}

.read_vpf <- function() {
  utils::read.csv(.gb_extdata("vpf.csv"), stringsAsFactors = FALSE)
}

.read_qaly <- function() {
  utils::read.csv(.gb_extdata("qaly.csv"), stringsAsFactors = FALSE)
}

.read_carbon <- function() {
  utils::read.csv(.gb_extdata("carbon_values.csv"), stringsAsFactors = FALSE)
}

# Validate numeric vector input
validate_numeric <- function(x, arg = "x", allow_na = FALSE, require_positive = FALSE) {
  if (!is.numeric(x)) {
    cli::cli_abort("{.arg {arg}} must be a numeric vector.")
  }
  if (!allow_na && anyNA(x)) {
    cli::cli_abort("{.arg {arg}} contains {.val NA} values.")
  }
  if (require_positive && any(x <= 0, na.rm = TRUE)) {
    cli::cli_abort("{.arg {arg}} must be strictly positive.")
  }
  invisible(x)
}

# Validate year vector
validate_year <- function(years, arg = "years", min_year = 0L, max_year = NULL) {
  validate_numeric(years, arg = arg)
  if (any(years < min_year)) {
    cli::cli_abort("{.arg {arg}} must be >= {min_year}.")
  }
  if (!is.null(max_year) && any(years > max_year)) {
    cli::cli_abort("{.arg {arg}} must be <= {max_year}.")
  }
  if (any(years != as.integer(years))) {
    cli::cli_abort("{.arg {arg}} must be integer-valued.")
  }
  invisible(years)
}

# GBP formatter for print methods. Charles's house style: "GBP 45m" / "GBP 250k".
.format_gbp <- function(x, digits = 1) {
  if (length(x) != 1L || !is.finite(x)) return(format(x))
  if (abs(x) >= 1e9) return(sprintf("GBP %.2fbn", x / 1e9))
  if (abs(x) >= 1e6) return(sprintf("GBP %.2fm", x / 1e6))
  if (abs(x) >= 1e3) return(sprintf("GBP %.1fk", x / 1e3))
  sprintf(paste0("GBP %.", digits, "f"), x)
}

#' Compare multiple appraisal options
#'
#' Builds a side-by-side comparison of two or more `gb_appraisal`
#' objects. Returns a `gb_comparison` with a ranked summary table
#' and the preferred option under both NPV and BCR criteria.
#'
#' @param ... `gb_appraisal` objects to compare. Pass as named
#'   arguments to control labels (e.g. `do_minimum = app1`).
#' @param by Character. Ranking criterion: one of `"npv"` (default)
#'   or `"bcr"`.
#'
#' @return A `gb_comparison` object: a list with class
#'   `c("gb_comparison", "list")` and elements `options`,
#'   `summary_table`, `preferred_npv`, `preferred_bcr`, `by`.
#'
#' @details
#' Every Green Book economic case must compare at least Do Minimum
#' against one or more Do Something options. `gb_compare()`
#' standardises the comparison: NPV, BCR, EANC, PV costs, PV benefits,
#' and rank under both NPV and BCR. The `summary()` method renders a
#' one-page table suitable for a Five Case Model economic case.
#'
#' @references
#' HM Treasury (2022). The Green Book: Central Government Guidance
#' on Appraisal and Evaluation, chapter on options analysis.
#'
#' @family appraisal
#' @seealso [gb_appraise()], [gb_economic_case()].
#'
#' @export
#' @examples
#' do_minimum <- gb_appraise(c(50, 0, 0, 0, 0), c(0, 20, 20, 20, 20))
#' do_max <- gb_appraise(c(150, 0, 0, 0, 0), c(0, 50, 50, 50, 50))
#' gb_compare(do_minimum = do_minimum, do_max = do_max)
gb_compare <- function(..., by = "npv") {
  by <- match.arg(by, c("npv", "bcr"))
  options <- list(...)
  if (length(options) < 2L) {
    cli::cli_abort("{.fn gb_compare} requires at least two appraisals.")
  }
  if (is.null(names(options)) || any(!nzchar(names(options)))) {
    names(options) <- paste0("option_", seq_along(options))
  }
  for (i in seq_along(options)) {
    if (!inherits(options[[i]], "gb_appraisal")) {
      cli::cli_abort("All inputs must be {.cls gb_appraisal} objects.")
    }
  }

  rows <- lapply(seq_along(options), function(i) {
    a <- options[[i]]
    horizon <- length(a$cashflow)
    eanc <- tryCatch(
      gb_eanc(a, years = horizon - 1L),
      error = function(e) NA_real_
    )
    data.frame(
      option = names(options)[i],
      npv = a$npv,
      bcr = a$bcr %||% NA_real_,
      eanc = eanc,
      pv_costs = a$pv_costs %||% NA_real_,
      pv_benefits = a$pv_benefits %||% NA_real_,
      stringsAsFactors = FALSE
    )
  })
  tbl <- do.call(rbind, rows)

  tbl$rank_npv <- rank(-tbl$npv, ties.method = "min", na.last = "keep")
  tbl$rank_bcr <- rank(-tbl$bcr, ties.method = "min", na.last = "keep")

  preferred_npv <- tbl$option[which.max(tbl$npv)]
  preferred_bcr <- if (any(!is.na(tbl$bcr))) tbl$option[which.max(tbl$bcr)] else NA_character_

  out <- list(
    options = options,
    summary_table = tbl,
    preferred_npv = preferred_npv,
    preferred_bcr = preferred_bcr,
    by = by
  )
  class(out) <- c("gb_comparison", "list")
  out
}

#' @export
print.gb_comparison <- function(x, ...) {
  cli::cli_h1("Green Book option comparison")
  cli::cli_text("{.val {length(x$options)}} options compared, ranked by {.field {x$by}}")
  cli::cli_text("Preferred (NPV): {.val {x$preferred_npv}}")
  if (!is.na(x$preferred_bcr)) {
    cli::cli_text("Preferred (BCR): {.val {x$preferred_bcr}}")
  }
  cat("\n")
  print(x$summary_table, row.names = FALSE)
  invisible(x)
}

#' @export
summary.gb_comparison <- function(object, ...) {
  cat("Green Book option comparison\n")
  cat("----------------------------\n")
  cat(sprintf("Options    : %d\n", length(object$options)))
  cat(sprintf("Ranked by  : %s\n", object$by))
  cat(sprintf("Preferred (NPV): %s\n", object$preferred_npv))
  if (!is.na(object$preferred_bcr)) {
    cat(sprintf("Preferred (BCR): %s\n", object$preferred_bcr))
  }
  cat("\n")
  print(object$summary_table, row.names = FALSE)
  invisible(object)
}

test_that("gb_compare returns gb_comparison with correct structure", {
  a <- gb_appraise(c(50, 0), c(0, 80))
  b <- gb_appraise(c(100, 0), c(0, 150))
  cmp <- gb_compare(a = a, b = b)
  expect_s3_class(cmp, "gb_comparison")
  expect_equal(nrow(cmp$summary_table), 2L)
  expect_true(all(c("option", "npv", "bcr", "rank_npv", "rank_bcr") %in%
                  names(cmp$summary_table)))
})

test_that("gb_compare ranks correctly by NPV", {
  a <- gb_appraise(c(50, 0), c(0, 80))   # NPV = -50 + 80/1.035 ≈ 27.3
  b <- gb_appraise(c(100, 0), c(0, 200)) # NPV = -100 + 200/1.035 ≈ 93.2
  cmp <- gb_compare(small = a, large = b)
  expect_equal(cmp$preferred_npv, "large")
})

test_that("gb_compare auto-labels unnamed options", {
  a <- gb_appraise(c(50, 0), c(0, 80))
  b <- gb_appraise(c(100, 0), c(0, 200))
  cmp <- gb_compare(a, b)
  expect_equal(cmp$summary_table$option, c("option_1", "option_2"))
})

test_that("gb_compare errors with fewer than 2 inputs", {
  a <- gb_appraise(c(50, 0), c(0, 80))
  expect_error(gb_compare(a))
})

test_that("gb_compare errors on non-appraisal input", {
  a <- gb_appraise(c(50, 0), c(0, 80))
  expect_error(gb_compare(a, "not an appraisal"))
})

test_that("gb_compare BCR ranking works when BCR available", {
  a <- gb_appraise(c(50, 0), c(0, 100))
  b <- gb_appraise(c(50, 0), c(0, 200))
  cmp <- gb_compare(low = a, high = b)
  expect_equal(cmp$preferred_bcr, "high")
})

test_that("gb_compare print method does not error", {
  a <- gb_appraise(c(50, 0), c(0, 80))
  b <- gb_appraise(c(100, 0), c(0, 200))
  cmp <- gb_compare(a = a, b = b)
  expect_no_error(capture.output(print(cmp)))
})

test_that("gb_economic_case wraps an appraisal", {
  app <- gb_appraise(c(100, 0), c(0, 200))
  ec <- gb_economic_case(app,
                          critical_success_factors = c("VfM", "Achievability"),
                          options_considered = c("Do nothing", "Do max"),
                          recommendation = "Do max")
  expect_s3_class(ec, "gb_economic_case")
  expect_equal(length(ec$critical_success_factors), 2L)
  expect_equal(ec$recommendation, "Do max")
})

test_that("gb_economic_case accepts a gb_comparison", {
  a <- gb_appraise(c(50, 0), c(0, 100))
  b <- gb_appraise(c(100, 0), c(0, 200))
  cmp <- gb_compare(a = a, b = b)
  ec <- gb_economic_case(cmp)
  expect_s3_class(ec, "gb_economic_case")
})

test_that("gb_economic_case errors on bad input", {
  expect_error(gb_economic_case("not an appraisal"))
})

test_that("gb_economic_case prints sections without errors", {
  app <- gb_appraise(c(100, 0), c(0, 200))
  ec <- gb_economic_case(app, recommendation = "Do max")
  expect_no_error(capture.output(print(ec)))
})

#' Track an appraisal across SOC, OBC, and FBC stages
#'
#' Builds a `gb_progression` showing how NPV, BCR, and key
#' parameters evolve across the three Green Book business case
#' stages: Strategic Outline Case (SOC), Outline Business Case
#' (OBC), and Full Business Case (FBC). Useful for both authoring
#' a multi-stage appraisal and reviewing how figures have moved
#' between gateway approvals.
#'
#' @param soc Strategic Outline Case `gb_appraisal`.
#' @param obc Outline Business Case `gb_appraisal`. Optional.
#' @param fbc Full Business Case `gb_appraisal`. Optional.
#'
#' @return A `gb_progression` object with elements `stages`,
#'   `evolution` (data frame), `delta_npv`, `delta_costs`.
#'
#' @details
#' At each stage, optimism bias mitigation typically increases as
#' project definition firms up. Cost estimates converge towards
#' base-cost reality; benefit estimates may also shift as evidence
#' accumulates. The `evolution` table makes the trajectory visible.
#'
#' @references HM Treasury (2018). Guide to Developing the Project
#'   Business Case (Green Book supplementary guidance).
#'
#' @family appraisal
#' @seealso [gb_appraise()].
#'
#' @export
#' @examples
#' soc <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30),
#'                    ob = "non_standard_buildings", ob_mitigation = 0)
#' obc <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30),
#'                    ob = "non_standard_buildings", ob_mitigation = 0.5)
#' fbc <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30),
#'                    ob = "non_standard_buildings", ob_mitigation = 0.9)
#' gb_progression(soc, obc, fbc)
gb_progression <- function(soc, obc = NULL, fbc = NULL) {
  if (!inherits(soc, "gb_appraisal")) {
    cli::cli_abort("{.arg soc} must be a {.cls gb_appraisal}.")
  }
  stages <- list(SOC = soc)
  if (!is.null(obc)) {
    if (!inherits(obc, "gb_appraisal")) {
      cli::cli_abort("{.arg obc} must be a {.cls gb_appraisal}.")
    }
    stages$OBC <- obc
  }
  if (!is.null(fbc)) {
    if (!inherits(fbc, "gb_appraisal")) {
      cli::cli_abort("{.arg fbc} must be a {.cls gb_appraisal}.")
    }
    stages$FBC <- fbc
  }

  rows <- lapply(seq_along(stages), function(i) {
    a <- stages[[i]]
    data.frame(
      stage = names(stages)[i],
      npv = a$npv,
      bcr = a$bcr %||% NA_real_,
      pv_costs = a$pv_costs %||% NA_real_,
      pv_benefits = a$pv_benefits %||% NA_real_,
      ob_pct = a$optimism_bias %||% NA_real_,
      stringsAsFactors = FALSE
    )
  })
  evolution <- do.call(rbind, rows)

  delta_npv <- if (length(stages) >= 2L) diff(evolution$npv) else numeric(0)
  delta_costs <- if (length(stages) >= 2L) diff(evolution$pv_costs) else numeric(0)

  out <- list(
    stages = stages,
    evolution = evolution,
    delta_npv = delta_npv,
    delta_costs = delta_costs
  )
  class(out) <- c("gb_progression", "list")
  out
}

#' @export
print.gb_progression <- function(x, ...) {
  cli::cli_h1("Business case progression")
  cli::cli_text("Stages: {.val {names(x$stages)}}")
  cat("\n")
  print(x$evolution, row.names = FALSE)
  if (length(x$delta_npv) > 0L) {
    cat("\n")
    cli::cli_text("NPV changes: {paste(sprintf('%+.1f', x$delta_npv), collapse = ' -> ')}")
  }
  invisible(x)
}

test_that("gb_progression works with all three stages", {
  soc <- gb_appraise(c(100, 0), c(0, 200), ob = 0.50, ob_mitigation = 0)
  obc <- gb_appraise(c(100, 0), c(0, 200), ob = 0.50, ob_mitigation = 0.5)
  fbc <- gb_appraise(c(100, 0), c(0, 200), ob = 0.50, ob_mitigation = 0.9)
  prog <- gb_progression(soc, obc, fbc)
  expect_s3_class(prog, "gb_progression")
  expect_equal(nrow(prog$evolution), 3L)
  expect_equal(prog$evolution$stage, c("SOC", "OBC", "FBC"))
})

test_that("gb_progression NPV improves as OB mitigation increases", {
  soc <- gb_appraise(c(100, 0), c(0, 200), ob = 0.50, ob_mitigation = 0)
  obc <- gb_appraise(c(100, 0), c(0, 200), ob = 0.50, ob_mitigation = 0.5)
  fbc <- gb_appraise(c(100, 0), c(0, 200), ob = 0.50, ob_mitigation = 0.9)
  prog <- gb_progression(soc, obc, fbc)
  expect_true(all(prog$delta_npv > 0))
})

test_that("gb_progression accepts SOC only", {
  soc <- gb_appraise(c(100, 0), c(0, 200))
  prog <- gb_progression(soc)
  expect_equal(nrow(prog$evolution), 1L)
  expect_length(prog$delta_npv, 0L)
})

test_that("gb_progression errors on non-appraisal input", {
  expect_error(gb_progression("nope"))
})

test_that("gb_progression print method does not error", {
  soc <- gb_appraise(c(100, 0), c(0, 200))
  obc <- gb_appraise(c(100, 0), c(0, 200))
  expect_no_error(capture.output(print(gb_progression(soc, obc))))
})

#' Tibble of the STPR schedule
#'
#' Returns the bundled STPR kinked schedule as a data frame: one row
#' per band, columns `year_from`, `year_to`, and (depending on
#' `schedule`) either all three rate variants or just the requested
#' one.
#'
#' @param schedule Optional. One of `"standard"`, `"health"`,
#'   `"catastrophic"`. If `NULL` (default), all three columns are
#'   returned.
#'
#' @return A data frame.
#'
#' @family lookups
#' @seealso [gb_stpr()], [gb_data_versions()].
#'
#' @export
#' @examples
#' gb_schedule_table()
#' gb_schedule_table("health")
gb_schedule_table <- function(schedule = NULL) {
  tbl <- .read_stpr()
  if (!is.null(schedule)) {
    schedule <- match.arg(schedule, c("standard", "health", "catastrophic"))
    tbl <- tbl[, c("year_from", "year_to", schedule)]
    names(tbl)[3] <- "rate"
  }
  tbl
}

#' Vintage of bundled parameter tables
#'
#' Returns a data frame describing the source and last-updated date
#' of every CSV bundled in `inst/extdata/`. Critical for
#' reproducibility: every appraisal can record which vintage of
#' Green Book parameters it used.
#'
#' @return A data frame with columns `dataset`, `source`,
#'   `last_updated`, `notes`.
#'
#' @family lookups
#' @seealso [gb_schedule_table()].
#'
#' @export
#' @examples
#' gb_data_versions()
gb_data_versions <- function() {
  .read_data_versions()
}

test_that("gb_schedule_table returns all bands by default", {
  tbl <- gb_schedule_table()
  expect_s3_class(tbl, "data.frame")
  expect_true(all(c("year_from", "year_to", "standard", "health", "catastrophic") %in% names(tbl)))
  expect_equal(nrow(tbl), 6L)
})

test_that("gb_schedule_table narrows columns when schedule supplied", {
  tbl <- gb_schedule_table("health")
  expect_named(tbl, c("year_from", "year_to", "rate"))
  expect_equal(tbl$rate[1], 0.015)
})

test_that("gb_schedule_table errors on invalid schedule", {
  expect_error(gb_schedule_table("nonsense"))
})

test_that("gb_data_versions returns vintage metadata", {
  v <- gb_data_versions()
  expect_s3_class(v, "data.frame")
  expect_true(all(c("dataset", "source", "last_updated", "notes") %in% names(v)))
  expect_true("stpr_schedule" %in% v$dataset)
  expect_true("gdp_deflator" %in% v$dataset)
})

#' Render an appraisal as a Five Case Model economic case
#'
#' Wraps a `gb_appraisal` with the structural sections HMT business
#' case guidance expects in the Economic Case (the second of the
#' five cases in the Five Case Model: Strategic, Economic,
#' Commercial, Financial, Management).
#'
#' @param appraisal A `gb_appraisal` (typically from `gb_appraise()`
#'   or `gb_compare()`).
#' @param critical_success_factors Character vector of CSFs.
#' @param options_considered Character vector of long-listed option
#'   names.
#' @param non_monetised_impacts Optional data frame with columns
#'   `impact`, `direction` (`"+"`/`"-"`), `materiality`
#'   (`"H"`/`"M"`/`"L"`), `notes`.
#' @param recommendation Optional character: the preferred option
#'   and rationale.
#' @param vfm_statement Optional character: the value-for-money
#'   judgment.
#'
#' @return A `gb_economic_case` object.
#'
#' @details
#' The Five Case Model is HM Treasury's standard structure for
#' business cases. The Economic Case is the part where Green Book
#' appraisal sits: monetised costs and benefits, non-monetised
#' impacts, switching values, sensitivity tests, value for money
#' judgment, recommended option. `gb_economic_case` wraps the
#' appraisal with the sections a reviewer expects to see.
#'
#' @references HM Treasury (2018). Guide to Developing the Project
#'   Business Case (Five Case Model).
#'
#' @family appraisal
#' @seealso [gb_appraise()], [gb_compare()].
#'
#' @export
#' @examples
#' app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
#' gb_economic_case(
#'   app,
#'   critical_success_factors = c("Strategic fit", "Value for money", "Achievability"),
#'   options_considered = c("Do nothing", "Do minimum", "Do maximum"),
#'   recommendation = "Do maximum: positive NPV and BCR > 1.5"
#' )
gb_economic_case <- function(appraisal,
                             critical_success_factors = NULL,
                             options_considered = NULL,
                             non_monetised_impacts = NULL,
                             recommendation = NULL,
                             vfm_statement = NULL) {
  if (!inherits(appraisal, "gb_appraisal") &&
      !inherits(appraisal, "gb_comparison")) {
    cli::cli_abort(
      "{.arg appraisal} must be a {.cls gb_appraisal} or {.cls gb_comparison}."
    )
  }

  out <- list(
    appraisal = appraisal,
    critical_success_factors = critical_success_factors,
    options_considered = options_considered,
    non_monetised_impacts = non_monetised_impacts,
    recommendation = recommendation,
    vfm_statement = vfm_statement
  )
  class(out) <- c("gb_economic_case", "list")
  out
}

#' @export
print.gb_economic_case <- function(x, ...) {
  cli::cli_h1("Economic Case (Five Case Model)")

  cli::cli_h2("Critical success factors")
  if (!is.null(x$critical_success_factors)) {
    for (csf in x$critical_success_factors) cli::cli_li(csf)
  } else {
    cli::cli_alert_info("Not specified")
  }

  cli::cli_h2("Options considered")
  if (!is.null(x$options_considered)) {
    for (o in x$options_considered) cli::cli_li(o)
  } else {
    cli::cli_alert_info("Not specified")
  }

  cli::cli_h2("Monetised summary")
  print(x$appraisal)

  if (!is.null(x$non_monetised_impacts)) {
    cli::cli_h2("Non-monetised impacts")
    print(x$non_monetised_impacts, row.names = FALSE)
  }

  if (!is.null(x$vfm_statement)) {
    cli::cli_h2("Value for money")
    cli::cli_text(x$vfm_statement)
  }

  if (!is.null(x$recommendation)) {
    cli::cli_h2("Recommendation")
    cli::cli_text(x$recommendation)
  }

  invisible(x)
}

#' Apply Marginal Excess Tax Burden to public expenditure
#'
#' Uplifts net public expenditure (or revenue raised) to reflect
#' the welfare cost of distortionary taxation. Default rate is
#' 20 percent per Green Book 2022 / 2026. The historic value (2003)
#' was 30 percent.
#'
#' @param values Numeric vector of expenditure values.
#' @param rate Numeric scalar. METB rate as a decimal. Default
#'   `0.20`.
#' @param vintage Optional character. One of `"2003"`, `"2018"`,
#'   `"2022"`, `"2026"`. If supplied, overrides `rate` with the
#'   bundled value for that vintage.
#'
#' @return A numeric vector the same length as `values`, with the
#'   METB uplift applied.
#'
#' @details
#' The METB captures the welfare loss arising from raising one extra
#' GBP of revenue through the tax system, beyond the GBP itself.
#' Estimates depend on the elasticity of taxable income, the marginal
#' tax rate, and the distortionary margin. The Green Book reduced the
#' headline value from 30 percent (2003) to 20 percent (2018) in
#' light of post-2008 evidence.
#'
#' @references HM Treasury (2022). The Green Book: Central Government
#'   Guidance on Appraisal and Evaluation, chapter on real terms and
#'   the marginal excess tax burden.
#'
#' @family adjustments
#' @seealso [gb_appraise()].
#'
#' @export
#' @examples
#' gb_metb(c(100, 200))
#' gb_metb(c(100, 200), vintage = "2003")
gb_metb <- function(values, rate = 0.20, vintage = NULL) {
  validate_numeric(values, "values")

  if (!is.null(vintage)) {
    metb_tbl <- .read_metb()
    if (!vintage %in% metb_tbl$vintage) {
      cli::cli_abort(
        "Unknown vintage. Available: {paste(metb_tbl$vintage, collapse = ', ')}."
      )
    }
    rate <- metb_tbl$rate[metb_tbl$vintage == vintage]
  }

  validate_numeric(rate, "rate")
  if (length(rate) != 1L) cli::cli_abort("{.arg rate} must be scalar.")
  if (rate < 0 || rate > 1) cli::cli_abort("{.arg rate} must be in [0, 1].")

  values * (1 + rate)
}

test_that("gb_metb default is 20 percent", {
  expect_equal(gb_metb(100), 120)
  expect_equal(gb_metb(c(100, 200)), c(120, 240))
})

test_that("gb_metb honours custom rate", {
  expect_equal(gb_metb(100, rate = 0.10), 110)
  expect_equal(gb_metb(100, rate = 0), 100)
})

test_that("gb_metb vintage lookup returns correct rate", {
  expect_equal(gb_metb(100, vintage = "2003"), 130)
  expect_equal(gb_metb(100, vintage = "2018"), 120)
  expect_equal(gb_metb(100, vintage = "2022"), 120)
  expect_equal(gb_metb(100, vintage = "2026"), 120)
})

test_that("gb_metb vintage overrides explicit rate", {
  # vintage takes precedence
  expect_equal(gb_metb(100, rate = 0.50, vintage = "2022"), 120)
})

test_that("gb_metb errors on unknown vintage", {
  expect_error(gb_metb(100, vintage = "1999"))
})

test_that("gb_metb errors on out-of-range rate", {
  expect_error(gb_metb(100, rate = -0.1))
  expect_error(gb_metb(100, rate = 1.5))
})

test_that("gb_metb errors on non-scalar rate", {
  expect_error(gb_metb(100, rate = c(0.1, 0.2)))
})

#' Full Green Book appraisal in one call
#'
#' Runs an end-to-end appraisal: optionally applies optimism bias to
#' costs, optionally applies METB, computes NPV under the kinked
#' STPR, and returns BCR alongside provenance.
#'
#' @param costs Numeric vector of cost values per period (real
#'   terms, base year fixed). Sign convention: enter as positive
#'   numbers; `gb_appraise()` handles the netting.
#' @param benefits Numeric vector of benefit values per period.
#'   Same length as `costs`.
#' @param years Optional integer vector of years. Defaults to
#'   `0:(length(costs) - 1)`.
#' @param schedule One of `"standard"`, `"health"`, `"catastrophic"`.
#' @param ob Optional. Either a category name (character) or a
#'   numeric percentage. If supplied, optimism bias uplift is
#'   applied to `costs`.
#' @param ob_dimension One of `"capex"` (default) or `"duration"`.
#'   Only used when `ob` is a category name.
#' @param ob_mitigation Numeric in `[0, 1]`. Mitigation fraction.
#' @param metb Logical. If `TRUE`, applies METB uplift to costs.
#' @param metb_rate Numeric. METB rate when `metb = TRUE`. Default
#'   `0.20` per Green Book 2022.
#' @param base_year Optional integer base year.
#' @param vintage Methodology vintage label. Default `"2022"`.
#'
#' @return A `gb_appraisal` object with extra fields `bcr`,
#'   `pv_costs`, `pv_benefits`, `costs`, `benefits`,
#'   `optimism_bias`, `metb_applied`.
#'
#' @references
#' HM Treasury (2026). The Green Book: Central Government Guidance
#' on Appraisal and Evaluation.
#'
#' @family appraisal
#' @seealso [gb_npv()], [gb_optimism_bias()], [gb_metb()],
#'   [gb_dist_weighted_npv()].
#'
#' @export
#' @examples
#' costs <- c(100, 50, 50, 0, 0, 0, 0, 0, 0, 0)
#' benefits <- c(0, 0, 0, 30, 30, 30, 30, 30, 30, 30)
#' app <- gb_appraise(costs, benefits, ob = "non_standard_buildings",
#'                    ob_mitigation = 0.5, base_year = 2024)
#' app
gb_appraise <- function(costs, benefits,
                        years = NULL,
                        schedule = "standard",
                        ob = NULL,
                        ob_dimension = "capex",
                        ob_mitigation = 0,
                        metb = FALSE,
                        metb_rate = 0.20,
                        base_year = NULL,
                        vintage = "2022") {
  validate_numeric(costs, "costs")
  validate_numeric(benefits, "benefits")
  if (length(costs) != length(benefits)) {
    cli::cli_abort("{.arg costs} and {.arg benefits} must have equal length.")
  }
  schedule <- match.arg(schedule, c("standard", "health", "catastrophic"))

  costs_adj <- costs
  ob_pct_used <- NULL
  if (!is.null(ob)) {
    if (is.character(ob)) {
      ob_pct_used <- gb_optimism_bias(ob, dimension = ob_dimension)
    } else if (is.numeric(ob) && length(ob) == 1L) {
      ob_pct_used <- ob
    } else {
      cli::cli_abort("{.arg ob} must be a category name or a numeric scalar.")
    }
    costs_adj <- gb_apply_ob(costs_adj, ob_pct_used, mitigation = ob_mitigation)
  }

  if (isTRUE(metb)) {
    costs_adj <- gb_metb(costs_adj, rate = metb_rate)
  }

  net <- benefits - costs_adj
  if (is.null(years)) years <- seq_along(net) - 1L

  pv_net <- gb_discount(net, years = years, schedule = schedule)
  pv_costs <- sum(gb_discount(costs_adj, years = years, schedule = schedule))
  pv_benefits <- sum(gb_discount(benefits, years = years, schedule = schedule))

  out <- list(
    npv = sum(pv_net),
    bcr = if (pv_costs > 0) pv_benefits / pv_costs else NA_real_,
    pv_costs = pv_costs,
    pv_benefits = pv_benefits,
    cashflow = net,
    costs = costs_adj,
    benefits = benefits,
    years = years,
    pv = pv_net,
    schedule = schedule,
    base_year = base_year,
    vintage = vintage,
    optimism_bias = ob_pct_used,
    metb_applied = isTRUE(metb)
  )
  class(out) <- c("gb_appraisal", "list")
  out
}

test_that("gb_risk_register computes expected loss correctly", {
  risks <- data.frame(
    id = c("R1", "R2"),
    description = c("Delay", "Overrun"),
    probability = c(0.3, 0.5),
    impact_gbp = c(100, 200),
    stringsAsFactors = FALSE
  )
  rr <- gb_risk_register(risks)
  expect_s3_class(rr, "gb_risk_register")
  expect_equal(rr$expected_value, 0.3 * 100 + 0.5 * 200)
  expect_equal(rr$total_exposure, 300)
})

test_that("gb_risk_register adds expected_loss column", {
  risks <- data.frame(
    id = "R1", description = "x",
    probability = 0.5, impact_gbp = 100,
    stringsAsFactors = FALSE
  )
  rr <- gb_risk_register(risks)
  expect_equal(rr$risks$expected_loss, 50)
})

test_that("gb_risk_register aggregates by category", {
  risks <- data.frame(
    id = c("R1", "R2", "R3"),
    description = c("a", "b", "c"),
    category = c("cost", "cost", "schedule"),
    probability = c(0.2, 0.3, 0.4),
    impact_gbp = c(100, 200, 50),
    stringsAsFactors = FALSE
  )
  rr <- gb_risk_register(risks)
  expect_false(is.null(rr$by_category))
  expect_equal(nrow(rr$by_category), 2L)
})

test_that("gb_risk_register risk-adjusts an appraisal NPV", {
  risks <- data.frame(
    id = "R1", description = "x",
    probability = 0.5, impact_gbp = 10,
    stringsAsFactors = FALSE
  )
  app <- gb_appraise(c(100, 0), c(0, 200))
  rr <- gb_risk_register(risks, appraisal = app)
  expect_equal(rr$risk_adjusted_npv, app$npv - 5)
})

test_that("gb_risk_register errors on missing columns", {
  bad <- data.frame(id = "R1", description = "x", stringsAsFactors = FALSE)
  expect_error(gb_risk_register(bad))
})

test_that("gb_risk_register errors on probabilities outside [0, 1]", {
  bad <- data.frame(
    id = "R1", description = "x",
    probability = 1.5, impact_gbp = 100,
    stringsAsFactors = FALSE
  )
  expect_error(gb_risk_register(bad))
})

test_that("gb_risk_register print method does not error", {
  risks <- data.frame(
    id = "R1", description = "x",
    probability = 0.5, impact_gbp = 100,
    stringsAsFactors = FALSE
  )
  expect_no_error(capture.output(print(gb_risk_register(risks))))
})

test_that("gb_validate passes a clean appraisal", {
  app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30),
                     base_year = 2024, vintage = "2022")
  v <- gb_validate(app)
  expect_s3_class(v, "gb_validation")
  expect_true(v$pass)
  expect_length(v$errors, 0L)
})

test_that("gb_validate flags missing base year as warning", {
  app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
  v <- gb_validate(app)
  warning_checks <- vapply(v$warnings, function(w) w$check, character(1))
  expect_true("base_year" %in% warning_checks)
})

test_that("gb_validate flags negative costs as warning", {
  app <- gb_appraise(c(-10, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
  v <- gb_validate(app)
  warning_checks <- vapply(v$warnings, function(w) w$check, character(1))
  expect_true("cost_sign" %in% warning_checks)
})

test_that("gb_validate flags unrecognised vintage", {
  app <- gb_appraise(c(100, 0), c(0, 200), vintage = "1999")
  v <- gb_validate(app)
  warning_checks <- vapply(v$warnings, function(w) w$check, character(1))
  expect_true("vintage" %in% warning_checks)
})

test_that("gb_validate flags long horizon as info", {
  costs <- c(100, rep(0, 119))
  benefits <- c(0, rep(5, 119))
  app <- gb_appraise(costs, benefits)
  v <- gb_validate(app)
  info_checks <- vapply(v$info, function(i) i$check, character(1))
  expect_true("horizon" %in% info_checks)
})

test_that("gb_validate errors on non-gb_appraisal input", {
  expect_error(gb_validate(list()))
  expect_error(gb_validate(123))
})

test_that("gb_validate print method does not error", {
  app <- gb_appraise(c(100, 0), c(0, 200))
  expect_no_error(capture.output(print(gb_validate(app))))
})

test_that("gb_appraise returns a gb_appraisal with BCR populated", {
  costs <- c(100, 50, 50, 0, 0, 0, 0, 0, 0, 0)
  benefits <- c(0, 0, 0, 30, 30, 30, 30, 30, 30, 30)
  app <- gb_appraise(costs, benefits)
  expect_s3_class(app, "gb_appraisal")
  expect_true(!is.null(app$bcr))
  expect_true(is.finite(app$bcr))
  expect_true(!is.null(app$pv_costs))
  expect_true(!is.null(app$pv_benefits))
})

test_that("gb_appraise NPV equals PV benefits minus PV costs", {
  costs <- c(100, 50, 50, 0, 0)
  benefits <- c(0, 0, 0, 50, 100)
  app <- gb_appraise(costs, benefits)
  expect_equal(app$npv, app$pv_benefits - app$pv_costs, tolerance = 1e-9)
})

test_that("gb_appraise BCR matches PV ratio", {
  costs <- c(100, 50, 50, 0, 0)
  benefits <- c(0, 0, 0, 50, 100)
  app <- gb_appraise(costs, benefits)
  expect_equal(app$bcr, app$pv_benefits / app$pv_costs, tolerance = 1e-9)
})

test_that("gb_appraise applies optimism bias by category", {
  costs <- c(100, 0)
  benefits <- c(0, 200)
  app_no_ob <- gb_appraise(costs, benefits)
  app_ob <- gb_appraise(costs, benefits, ob = "non_standard_buildings")
  expect_lt(app_ob$npv, app_no_ob$npv)
  expect_equal(app_ob$optimism_bias, 0.51)
})

test_that("gb_appraise applies optimism bias as numeric", {
  costs <- c(100, 0)
  benefits <- c(0, 200)
  app <- gb_appraise(costs, benefits, ob = 0.30)
  expect_equal(app$optimism_bias, 0.30)
})

test_that("gb_appraise applies OB mitigation", {
  costs <- c(100, 0)
  benefits <- c(0, 200)
  full_ob <- gb_appraise(costs, benefits, ob = 0.50)
  half_ob <- gb_appraise(costs, benefits, ob = 0.50, ob_mitigation = 0.5)
  expect_lt(full_ob$npv, half_ob$npv)
})

test_that("gb_appraise applies METB to costs", {
  costs <- c(100, 0)
  benefits <- c(0, 200)
  app_no_metb <- gb_appraise(costs, benefits)
  app_metb <- gb_appraise(costs, benefits, metb = TRUE)
  expect_lt(app_metb$npv, app_no_metb$npv)
  expect_true(app_metb$metb_applied)
})

test_that("gb_appraise errors on length mismatch", {
  expect_error(gb_appraise(c(100, 50), c(30, 30, 30)))
})

test_that("gb_appraise errors on invalid ob input", {
  expect_error(gb_appraise(c(100, 0), c(0, 200), ob = list(0.5)))
})

test_that("gb_appraise stores schedule, base year, vintage", {
  app <- gb_appraise(c(100, 0), c(0, 200), schedule = "health",
                     base_year = 2024, vintage = "2026")
  expect_equal(app$schedule, "health")
  expect_equal(app$base_year, 2024)
  expect_equal(app$vintage, "2026")
})

test_that("gb_appraise summary prints BCR and OB", {
  app <- gb_appraise(c(100, 50), c(0, 200), ob = "standard_buildings", metb = TRUE)
  out <- capture.output(summary(app))
  expect_true(any(grepl("BCR", out)))
  expect_true(any(grepl("Optimism bias", out)))
  expect_true(any(grepl("METB", out)))
})

#' GDP deflator factor between two years
#'
#' Returns the multiplicative factor to convert a value expressed in
#' year `from` prices to year `to` prices, using the bundled UK GDP
#' deflator at market prices (ONS series). Multiply a nominal value
#' from year `from` by this factor to express it in year `to`.
#'
#' @param from Integer year of the input value.
#' @param to Integer year to express the value in.
#' @param source Character. `"bundled"` uses the CSV shipped in
#'   `inst/extdata/`. Future versions will accept `"inflateR"` to use
#'   the live ONS series via the `inflateR` package.
#'
#' @return A numeric scalar: the deflator factor.
#'
#' @references Office for National Statistics. GDP deflator at market
#'   prices (series YBGB), updated quarterly. HM Treasury publishes
#'   the deflator series alongside the OBR forecast.
#'
#' @family deflator
#' @seealso [gb_real()], [gb_rebase()].
#'
#' @export
#' @examples
#' gb_deflator(from = 2020, to = 2024)
gb_deflator <- function(from, to, source = "bundled") {
  source <- match.arg(source, c("bundled"))
  if (length(from) != 1L || length(to) != 1L) {
    cli::cli_abort("{.arg from} and {.arg to} must be scalar.")
  }
  validate_year(from, "from", min_year = 1900L)
  validate_year(to, "to", min_year = 1900L)

  defl <- .read_deflator()
  rng <- range(defl$year)
  if (!(from %in% defl$year) || !(to %in% defl$year)) {
    cli::cli_abort(
      "{.arg from}/{.arg to} ({from}, {to}) outside bundled deflator range ({rng[1]} to {rng[2]})."
    )
  }
  defl$value[defl$year == to] / defl$value[defl$year == from]
}

#' Convert nominal values to real
#'
#' Converts nominal values at year-of-occurrence prices to real values
#' at a chosen base year, using the bundled GDP deflator.
#'
#' @param nominal_values Numeric vector of nominal values.
#' @param year Integer scalar or vector matching `nominal_values`,
#'   giving the year at which each value is expressed in nominal terms.
#' @param base_year Integer scalar: the base year to convert to.
#' @param source Character. `"bundled"` only in v0.1.0.
#'
#' @return A numeric vector of real values, in `base_year` prices.
#'
#' @references HM Treasury (2022). The Green Book, chapter on real
#'   terms and price-base alignment.
#'
#' @family deflator
#' @seealso [gb_deflator()], [gb_rebase()].
#'
#' @export
#' @examples
#' gb_real(nominal_values = c(100, 110, 120),
#'         year = 2020:2022,
#'         base_year = 2024)
gb_real <- function(nominal_values, year, base_year, source = "bundled") {
  validate_numeric(nominal_values, "nominal_values")
  if (length(year) == 1L) year <- rep_len(year, length(nominal_values))
  if (length(year) != length(nominal_values)) {
    cli::cli_abort("{.arg year} must be scalar or same length as {.arg nominal_values}.")
  }
  validate_year(base_year, "base_year", min_year = 1900L)
  if (length(base_year) != 1L) cli::cli_abort("{.arg base_year} must be scalar.")

  factors <- vapply(
    year,
    function(y) gb_deflator(from = y, to = base_year, source = source),
    numeric(1)
  )
  nominal_values * factors
}

#' Rebase a real-terms series to a different base year
#'
#' Multiplies values currently in `from`-year real prices by the
#' deflator factor to express them in `to`-year real prices.
#'
#' @param values Numeric vector of real-terms values.
#' @param from Integer base year of the input series.
#' @param to Integer target base year.
#'
#' @return A numeric vector of values in `to`-year real prices.
#'
#' @references HM Treasury (2022). The Green Book, chapter on real
#'   terms and price-base alignment.
#'
#' @family deflator
#' @seealso [gb_deflator()], [gb_real()].
#'
#' @export
#' @examples
#' gb_rebase(c(100, 200, 300), from = 2020, to = 2024)
gb_rebase <- function(values, from, to) {
  validate_numeric(values, "values")
  values * gb_deflator(from = from, to = to)
}

test_that("gb_deflator: same year returns 1", {
  expect_equal(gb_deflator(2020, 2020), 1)
  expect_equal(gb_deflator(2024, 2024), 1)
})

test_that("gb_deflator from past to future is > 1", {
  expect_gt(gb_deflator(2020, 2024), 1)
})

test_that("gb_deflator from future to past is < 1", {
  expect_lt(gb_deflator(2024, 2020), 1)
})

test_that("gb_deflator round-trip is identity", {
  f1 <- gb_deflator(2020, 2024)
  f2 <- gb_deflator(2024, 2020)
  expect_equal(f1 * f2, 1, tolerance = 1e-12)
})

test_that("gb_deflator errors on out-of-range years", {
  expect_error(gb_deflator(1800, 2024))
  expect_error(gb_deflator(2024, 2200), "outside bundled deflator")
})

test_that("gb_deflator errors on non-scalar input", {
  expect_error(gb_deflator(c(2020, 2021), 2024), "must be scalar")
  expect_error(gb_deflator(2020, c(2024, 2025)), "must be scalar")
})

test_that("gb_real with scalar year converts uniformly", {
  vals <- c(100, 200, 300)
  out <- gb_real(vals, year = 2020, base_year = 2024)
  factor <- gb_deflator(2020, 2024)
  expect_equal(out, vals * factor, tolerance = 1e-12)
})

test_that("gb_real with vector year applies element-wise", {
  vals <- c(100, 110, 120)
  yrs <- 2020:2022
  out <- gb_real(vals, year = yrs, base_year = 2024)
  expected <- mapply(function(v, y) v * gb_deflator(y, 2024), vals, yrs)
  expect_equal(out, expected, tolerance = 1e-12)
})

test_that("gb_real errors on length mismatch", {
  expect_error(
    gb_real(c(100, 200, 300), year = c(2020, 2021), base_year = 2024),
    "scalar or same length"
  )
})

test_that("gb_rebase equals values times deflator", {
  vals <- c(100, 200, 300)
  out <- gb_rebase(vals, from = 2020, to = 2024)
  expect_equal(out, vals * gb_deflator(2020, 2024), tolerance = 1e-12)
})

test_that("gb_rebase round-trip is identity", {
  vals <- c(100, 200, 300)
  out <- gb_rebase(gb_rebase(vals, from = 2020, to = 2024), from = 2024, to = 2020)
  expect_equal(out, vals, tolerance = 1e-12)
})

#' Build a risk register with monetised exposure
#'
#' Creates a `gb_risk_register` from a data frame of project risks.
#' Computes expected loss per risk (probability times impact) and
#' aggregate exposure. Optionally applied to a `gb_appraisal` to
#' produce a risk-adjusted NPV.
#'
#' @param risks A data frame with columns `id`, `description`,
#'   `probability` (numeric in `[0, 1]`), `impact_gbp` (numeric).
#'   Optional columns: `category`, `mitigation` (character).
#' @param appraisal Optional `gb_appraisal`. If supplied, returns a
#'   risk-adjusted NPV: `appraisal$npv - sum(probability * impact)`.
#'
#' @return A `gb_risk_register` object.
#'
#' @details
#' HM Treasury business case guidance requires a risk register with
#' monetised exposure for the OBC and FBC stages. `gb_risk_register`
#' standardises the structure: every risk has an id, a probability,
#' a monetary impact, and an expected loss. Aggregation is by category
#' if the column is present.
#'
#' @references HM Treasury (2018). The Orange Book: Management of
#'   Risk - Principles and Concepts.
#'
#' @family appraisal
#' @seealso [gb_appraise()].
#'
#' @export
#' @examples
#' risks <- data.frame(
#'   id = c("R1", "R2", "R3"),
#'   description = c("Planning delay", "Cost overrun", "Lower demand"),
#'   category = c("schedule", "cost", "demand"),
#'   probability = c(0.30, 0.50, 0.20),
#'   impact_gbp = c(2e6, 5e6, 10e6)
#' )
#' gb_risk_register(risks)
gb_risk_register <- function(risks, appraisal = NULL) {
  if (!is.data.frame(risks)) {
    cli::cli_abort("{.arg risks} must be a data frame.")
  }
  required <- c("id", "description", "probability", "impact_gbp")
  missing <- setdiff(required, names(risks))
  if (length(missing) > 0L) {
    cli::cli_abort("{.arg risks} is missing column(s): {paste(missing, collapse = ', ')}.")
  }
  validate_numeric(risks$probability, "probability")
  validate_numeric(risks$impact_gbp, "impact_gbp")
  if (any(risks$probability < 0 | risks$probability > 1)) {
    cli::cli_abort("{.arg probability} must be in [0, 1].")
  }

  risks$expected_loss <- risks$probability * risks$impact_gbp
  expected_value <- sum(risks$expected_loss)
  total_exposure <- sum(risks$impact_gbp)

  by_category <- NULL
  if ("category" %in% names(risks)) {
    by_category <- stats::aggregate(
      cbind(impact_gbp, expected_loss) ~ category,
      data = risks,
      FUN = sum
    )
  }

  risk_adjusted_npv <- NULL
  if (!is.null(appraisal)) {
    if (!inherits(appraisal, "gb_appraisal")) {
      cli::cli_abort("{.arg appraisal} must be a {.cls gb_appraisal}.")
    }
    risk_adjusted_npv <- appraisal$npv - expected_value
  }

  out <- list(
    risks = risks,
    expected_value = expected_value,
    total_exposure = total_exposure,
    by_category = by_category,
    risk_adjusted_npv = risk_adjusted_npv
  )
  class(out) <- c("gb_risk_register", "list")
  out
}

#' @export
print.gb_risk_register <- function(x, ...) {
  n_risks <- nrow(x$risks)
  exp_str <- .format_gbp(x$expected_value)
  exposure_str <- .format_gbp(x$total_exposure)
  cli::cli_h1("Risk register")
  cli::cli_text("Risks identified: {.val {n_risks}}")
  cli::cli_text("Expected loss (sum of probability x impact): {.val {exp_str}}")
  cli::cli_text("Maximum exposure (sum of impacts): {.val {exposure_str}}")
  if (!is.null(x$risk_adjusted_npv)) {
    adj_str <- .format_gbp(x$risk_adjusted_npv)
    cli::cli_text("Risk-adjusted NPV: {.val {adj_str}}")
  }
  cat("\n")
  print(x$risks[, c("id", "description", "probability", "impact_gbp", "expected_loss")],
        row.names = FALSE)
  if (!is.null(x$by_category)) {
    cat("\nBy category:\n")
    print(x$by_category, row.names = FALSE)
  }
  invisible(x)
}

#' DESNZ carbon value for appraisal
#'
#' Returns the DESNZ Carbon Value for Appraisal in GBP per tonne of
#' CO2-equivalent at the published 2022 base-year prices, from the
#' November 2023 valuation. Three scenarios: low, central, high.
#'
#' @param year Integer scalar or vector. Year of the emission. Must
#'   be within the bundled DESNZ range.
#' @param scenario One of `"low"`, `"central"` (default), `"high"`.
#' @param base_year Optional integer to rebase the value via
#'   `gb_deflator()`. If `NULL`, the published 2022 base year is
#'   used.
#'
#' @return Numeric vector of GBP per tCO2e values.
#'
#' @details
#' DESNZ moved to a single consolidated series in November 2023,
#' superseding the historical traded / non-traded split. Values
#' are in 2022 prices. The November 2023 publication covers 2010
#' to 2100; the bundled subset is 2020 to 2050.
#'
#' @references DESNZ (2023). Valuation of Energy Use and Greenhouse
#'   Gas Emissions for Appraisal (November 2023). Data tables 1-19,
#'   Table 3.
#'
#' @family carbon
#' @seealso [gb_carbon_npv()].
#'
#' @export
#' @examples
#' gb_carbon_value(2024)
#' gb_carbon_value(2020:2030)
#' gb_carbon_value(2030, scenario = "high")
gb_carbon_value <- function(year, scenario = "central", base_year = NULL) {
  scenario <- match.arg(scenario, c("low", "central", "high"))
  validate_year(year, "year", min_year = 1900L)

  tbl <- .read_carbon()
  rng <- range(tbl$year)
  if (any(year < rng[1]) || any(year > rng[2])) {
    cli::cli_abort(
      "{.arg year} outside bundled DESNZ carbon value range ({rng[1]} to {rng[2]})."
    )
  }

  rows <- tbl$scenario == scenario
  sub <- tbl[rows, ]
  pub_base <- sub$base_year[1]

  out <- vapply(year, function(y) {
    val <- sub$value_gbp_per_tco2e[sub$year == y]
    if (length(val) == 0L) {
      cli::cli_abort("No bundled value for year {.val {y}}.")
    }
    val
  }, numeric(1))

  if (!is.null(base_year) && base_year != pub_base) {
    out <- out * gb_deflator(pub_base, base_year)
  }
  out
}

#' Net present value of an emissions path
#'
#' Multiplies an emissions vector (tCO2e per year) by the DESNZ
#' carbon value at each year, then discounts under the kinked STPR.
#' Returns a `gb_appraisal` object.
#'
#' @param emissions Numeric vector of emissions per year, in tCO2e
#'   (positive = emitted, negative = avoided / abated).
#' @param years Integer vector of years matching `emissions`.
#' @param scenario One of `"low"`, `"central"` (default), `"high"`.
#' @param schedule One of `"standard"` (default), `"health"`,
#'   `"catastrophic"`.
#' @param base_year Optional integer base year for monetary values
#'   (e.g. `2024`). If supplied, carbon values are rebased via
#'   `gb_deflator()`.
#' @param sign Character. `"cost"` (default) treats positive
#'   `emissions` as a cost (negative cashflow); `"benefit"` treats
#'   positive as avoided emissions (positive cashflow).
#'
#' @return A `gb_appraisal` object.
#'
#' @references DESNZ (2023). Valuation of Energy Use and Greenhouse
#'   Gas Emissions for Appraisal (November 2023). HM Treasury (2022).
#'   The Green Book, on environmental valuation.
#'
#' @family carbon
#' @seealso [gb_carbon_value()], [gb_npv()].
#'
#' @export
#' @examples
#' # 100 tCO2e emitted each year from 2024 to 2030
#' emissions <- rep(100, 7)
#' years <- 2024:2030
#' gb_carbon_npv(emissions, years, base_year = 2024)
gb_carbon_npv <- function(emissions, years,
                          scenario = "central",
                          schedule = "standard",
                          base_year = NULL,
                          sign = "cost") {
  validate_numeric(emissions, "emissions")
  validate_year(years, "years", min_year = 1900L)
  if (length(emissions) != length(years)) {
    cli::cli_abort("{.arg emissions} and {.arg years} must have equal length.")
  }
  sign <- match.arg(sign, c("cost", "benefit"))

  values_per_t <- gb_carbon_value(years, scenario = scenario,
                                  base_year = base_year)
  cashflow <- emissions * values_per_t
  if (sign == "cost") cashflow <- -cashflow

  rel_years <- years - min(years)
  app <- gb_npv(cashflow, years = rel_years, schedule = schedule,
                base_year = base_year %||% min(years))
  app$emissions <- emissions
  app$years_calendar <- years
  app$carbon_scenario <- scenario
  app$emissions_sign <- sign
  app
}

test_that("gb_carbon_value matches DESNZ Nov 2023 Table 3 published values", {
  # DESNZ Nov 2023, central path, 2022 GBP/tCO2e
  expect_equal(gb_carbon_value(2020), 253.19)
  expect_equal(gb_carbon_value(2024), 268.97)
  expect_equal(gb_carbon_value(2030), 294.50)
  expect_equal(gb_carbon_value(2050), 397.54)
})

test_that("gb_carbon_value low and high scenarios match Table 3", {
  expect_equal(gb_carbon_value(2020, "low"), 126.60)
  expect_equal(gb_carbon_value(2020, "high"), 379.79)
  expect_equal(gb_carbon_value(2050, "low"), 198.77)
  expect_equal(gb_carbon_value(2050, "high"), 596.31)
})

test_that("gb_carbon_value low < central < high at every year", {
  for (y in c(2020, 2030, 2040, 2050)) {
    expect_lt(gb_carbon_value(y, "low"), gb_carbon_value(y, "central"))
    expect_gt(gb_carbon_value(y, "high"), gb_carbon_value(y, "central"))
  }
})

test_that("gb_carbon_value is monotonically increasing over time", {
  out <- gb_carbon_value(2020:2050)
  expect_true(all(diff(out) > 0))
})

test_that("gb_carbon_value supports vectorised year input", {
  out <- gb_carbon_value(c(2020, 2030, 2050))
  expect_length(out, 3L)
  expect_equal(out[1], 253.19)
  expect_equal(out[3], 397.54)
})

test_that("gb_carbon_value rebases via deflator", {
  v_2022 <- gb_carbon_value(2025, base_year = 2022)
  v_2024 <- gb_carbon_value(2025, base_year = 2024)
  factor <- gb_deflator(2022, 2024)
  expect_equal(v_2024, v_2022 * factor, tolerance = 1e-6)
})

test_that("gb_carbon_value errors outside bundled range", {
  expect_error(gb_carbon_value(2019))
  expect_error(gb_carbon_value(2055))
})

test_that("gb_carbon_value errors on bad scenario", {
  expect_error(gb_carbon_value(2030, scenario = "wrong"))
})

test_that("gb_carbon_npv returns gb_appraisal with carbon-specific fields", {
  app <- gb_carbon_npv(rep(100, 7), 2024:2030)
  expect_s3_class(app, "gb_appraisal")
  expect_true(!is.null(app$emissions))
  expect_true(!is.null(app$years_calendar))
  expect_true(!is.null(app$carbon_scenario))
})

test_that("gb_carbon_npv treats positive emissions as cost by default", {
  app_cost <- gb_carbon_npv(rep(100, 5), 2024:2028)
  expect_lt(app_cost$npv, 0)
})

test_that("gb_carbon_npv with sign='benefit' inverts the cashflow", {
  app_cost <- gb_carbon_npv(rep(100, 5), 2024:2028, sign = "cost")
  app_benefit <- gb_carbon_npv(rep(100, 5), 2024:2028, sign = "benefit")
  expect_equal(app_cost$npv, -app_benefit$npv, tolerance = 1e-9)
})

test_that("gb_carbon_npv errors on length mismatch", {
  expect_error(gb_carbon_npv(c(100, 200), 2024:2026))
})

#' Lint a Green Book appraisal for common errors
#'
#' Inspects a `gb_appraisal` for sign-convention errors, base-year
#' alignment, schedule plausibility, and other common authoring
#' mistakes. Returns a structured report classifying each check as
#' pass, warning, or error.
#'
#' @param appraisal A `gb_appraisal` object.
#'
#' @return A `gb_validation` object with elements `pass`, `errors`,
#'   `warnings`, `info`, and `n_checks`.
#'
#' @details
#' Checks performed:
#' - Cashflow length matches `years` length
#' - Schedule is one of the valid options
#' - Base year is plausible (1900 to 2100)
#' - Vintage matches a recognised Green Book edition
#' - If `costs` and `benefits` are present (from `gb_appraise()`),
#'   costs are non-negative and benefits are non-negative
#' - NPV is finite (no NaN / Inf)
#' - PV costs and PV benefits are consistent with NPV
#'
#' Warnings flag suspicious patterns: missing base year, unusual
#' optimism bias values, METB outside 0 to 50 percent, very long
#' horizons (> 100 years).
#'
#' @family appraisal
#' @seealso [gb_appraise()].
#'
#' @export
#' @examples
#' app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
#' gb_validate(app)
gb_validate <- function(appraisal) {
  if (!inherits(appraisal, "gb_appraisal")) {
    cli::cli_abort("{.arg appraisal} must be a {.cls gb_appraisal}.")
  }

  errors <- list()
  warnings <- list()
  info <- list()

  add <- function(level, check, msg) {
    item <- list(check = check, message = msg)
    if (level == "error") errors[[length(errors) + 1L]] <<- item
    else if (level == "warning") warnings[[length(warnings) + 1L]] <<- item
    else info[[length(info) + 1L]] <<- item
  }

  if (length(appraisal$cashflow) != length(appraisal$years)) {
    add("error", "length", "cashflow and years differ in length")
  }

  valid_schedules <- c("standard", "health", "catastrophic")
  if (!appraisal$schedule %in% valid_schedules) {
    add("error", "schedule",
        sprintf("schedule '%s' is not one of: %s",
                appraisal$schedule, paste(valid_schedules, collapse = ", ")))
  }

  if (!is.null(appraisal$base_year)) {
    by <- appraisal$base_year
    if (!is.numeric(by) || by < 1900 || by > 2100) {
      add("error", "base_year",
          sprintf("base_year %s is outside 1900 to 2100", by))
    }
  } else {
    add("warning", "base_year",
        "base_year not set; appraisal is not self-describing for price-base alignment")
  }

  if (!is.null(appraisal$vintage)) {
    if (!appraisal$vintage %in% c("2003", "2018", "2022", "2026")) {
      add("warning", "vintage",
          sprintf("vintage '%s' is not a recognised Green Book edition", appraisal$vintage))
    }
  }

  if (!is.null(appraisal$costs)) {
    if (any(appraisal$costs < 0, na.rm = TRUE)) {
      add("warning", "cost_sign",
          "costs contain negative values; gb_appraise expects positive costs")
    }
  }
  if (!is.null(appraisal$benefits)) {
    if (any(appraisal$benefits < 0, na.rm = TRUE)) {
      add("warning", "benefit_sign",
          "benefits contain negative values")
    }
  }

  if (!is.finite(appraisal$npv)) {
    add("error", "npv_finite", "NPV is not finite (NaN or Inf)")
  }

  if (!is.null(appraisal$pv_costs) && !is.null(appraisal$pv_benefits)) {
    expected_npv <- appraisal$pv_benefits - appraisal$pv_costs
    if (abs(expected_npv - appraisal$npv) > 1e-6) {
      add("error", "npv_consistency",
          sprintf("NPV (%g) does not equal PV benefits (%g) minus PV costs (%g)",
                  appraisal$npv, appraisal$pv_benefits, appraisal$pv_costs))
    }
  }

  if (!is.null(appraisal$optimism_bias)) {
    ob <- appraisal$optimism_bias
    if (ob < 0 || ob > 3) {
      add("warning", "ob_range",
          sprintf("optimism bias %.1f%% is outside 0%% to 300%%; check for unit error",
                  100 * ob))
    }
  }

  horizon <- length(appraisal$cashflow)
  if (horizon > 100) {
    add("info", "horizon",
        sprintf("horizon is %d years; consider a lower-rate sensitivity test for long-horizon impacts",
                horizon))
  }

  out <- list(
    pass = length(errors) == 0L,
    errors = errors,
    warnings = warnings,
    info = info,
    n_checks = 8L
  )
  class(out) <- c("gb_validation", "list")
  out
}

#' @export
print.gb_validation <- function(x, ...) {
  if (x$pass) {
    cli::cli_alert_success("Validation passed: {.val {x$n_checks}} checks, no errors")
  } else {
    cli::cli_alert_danger("Validation failed: {.val {length(x$errors)}} error(s)")
  }
  if (length(x$errors) > 0L) {
    cli::cli_h2("Errors")
    for (e in x$errors) {
      cli::cli_bullets(c("x" = "{e$check}: {e$message}"))
    }
  }
  if (length(x$warnings) > 0L) {
    cli::cli_h2("Warnings")
    for (w in x$warnings) {
      cli::cli_bullets(c("!" = "{w$check}: {w$message}"))
    }
  }
  if (length(x$info) > 0L) {
    cli::cli_h2("Info")
    for (i in x$info) {
      cli::cli_bullets(c("i" = "{i$check}: {i$message}"))
    }
  }
  invisible(x)
}

#' greenbook: HM Treasury Green Book Cost-Benefit Analysis Primitives
#'
#' Implements cost-benefit analysis primitives from HM Treasury Green Book
#' guidance: the kinked Social Time Preference Rate (STPR), discount factors,
#' net present value, equivalent annual cost, and GDP-deflator rebasing.
#' Pure computation, no network access. Bundled parameter tables carry
#' vintage metadata for reproducibility.
#'
#' @keywords internal
"_PACKAGE"

#' @importFrom cli cli_abort cli_inform cli_warn cli_text cli_h1
#' @importFrom stats median
#' @importFrom utils read.csv
NULL

#' One-page headline summary of an appraisal
#'
#' Returns the headline numbers a Green Book reviewer or steering
#' committee expects: NPV, BCR, EANC, payback period, optimism bias
#' applied, vintage. Suitable for a slide or executive summary.
#'
#' @param appraisal A `gb_appraisal` object.
#'
#' @return A `gb_headline` object with elements `npv`, `bcr`,
#'   `eanc`, `payback_year`, `optimism_bias`, `metb_applied`,
#'   `vintage`, `base_year`, `horizon`.
#'
#' @family reporting
#' @seealso [gb_appraise()], [gb_to_latex()], [gb_to_excel()].
#'
#' @export
#' @examples
#' app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
#' gb_headline(app)
gb_headline <- function(appraisal) {
  if (!inherits(appraisal, "gb_appraisal")) {
    cli::cli_abort("{.arg appraisal} must be a {.cls gb_appraisal}.")
  }
  horizon <- length(appraisal$cashflow)
  cumulative <- cumsum(appraisal$pv)
  payback_year <- if (any(cumulative > 0)) {
    appraisal$years[which(cumulative > 0)[1]]
  } else {
    NA_integer_
  }
  eanc <- tryCatch(
    gb_eanc(appraisal, years = horizon - 1L),
    error = function(e) NA_real_
  )

  out <- list(
    npv = appraisal$npv,
    bcr = appraisal$bcr %||% NA_real_,
    eanc = eanc,
    payback_year = payback_year,
    optimism_bias = appraisal$optimism_bias,
    metb_applied = isTRUE(appraisal$metb_applied),
    vintage = appraisal$vintage,
    base_year = appraisal$base_year,
    horizon = horizon
  )
  class(out) <- c("gb_headline", "list")
  out
}

#' @export
print.gb_headline <- function(x, ...) {
  npv_str <- .format_gbp(x$npv)
  bcr_str <- if (!is.na(x$bcr)) sprintf("%.2f", x$bcr) else NA
  eanc_str <- if (!is.na(x$eanc)) .format_gbp(x$eanc) else NA
  ob_str <- if (!is.null(x$optimism_bias))
    sprintf("%.0f%%", 100 * x$optimism_bias) else NA
  cli::cli_h1("Appraisal headline")
  cli::cli_text("NPV: {.val {npv_str}}")
  if (!is.na(x$bcr)) cli::cli_text("BCR: {.val {bcr_str}}")
  if (!is.na(x$eanc)) cli::cli_text("EANC: {.val {eanc_str}}")
  if (!is.na(x$payback_year)) {
    cli::cli_text("Payback: year {.val {x$payback_year}}")
  } else {
    cli::cli_text("Payback: never (cumulative PV does not turn positive)")
  }
  if (!is.null(x$optimism_bias)) {
    cli::cli_text("Optimism bias applied: {ob_str}")
  }
  if (x$metb_applied) cli::cli_text("METB applied to costs")
  cli::cli_text("Horizon: {.val {x$horizon}} years")
  if (!is.null(x$base_year)) cli::cli_text("Base year: {.val {x$base_year}}")
  cli::cli_text("Vintage: Green Book {.val {x$vintage}}")
  invisible(x)
}

#' Cost per unit delivered
#'
#' Computes a cost-effectiveness ratio: total real cost (PV) per
#' unit of monetised or non-monetised output. Standard cross-scheme
#' comparator used by HM Treasury reviewers.
#'
#' @param appraisal A `gb_appraisal` object.
#' @param units_delivered Numeric scalar. The total quantity of the
#'   output being delivered (e.g. QALYs gained, tCO2e abated,
#'   WELLBYs added, jobs created).
#' @param unit Character. Label for the unit (used in print).
#'
#' @return Numeric scalar: PV cost / units delivered, in GBP per
#'   unit.
#'
#' @family reporting
#' @seealso [gb_appraise()].
#'
#' @export
#' @examples
#' app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 0, 0, 0, 0))
#' gb_cost_per_unit(app, units_delivered = 50, unit = "QALY")
gb_cost_per_unit <- function(appraisal, units_delivered, unit = "unit") {
  if (!inherits(appraisal, "gb_appraisal")) {
    cli::cli_abort("{.arg appraisal} must be a {.cls gb_appraisal}.")
  }
  validate_numeric(units_delivered, "units_delivered", require_positive = TRUE)
  if (length(units_delivered) != 1L) {
    cli::cli_abort("{.arg units_delivered} must be scalar.")
  }
  pv_costs <- appraisal$pv_costs %||% sum(pmax(-appraisal$pv, 0))
  pv_costs / units_delivered
}

#' Render an appraisal as a LaTeX table
#'
#' @param appraisal A `gb_appraisal` object.
#' @param caption Optional table caption.
#' @param label Optional LaTeX label for cross-referencing.
#'
#' @return A character scalar containing a LaTeX `tabular`
#'   environment. Wrap in `\\begin{table}...\\end{table}` for a
#'   floating table.
#'
#' @family reporting
#' @seealso [gb_to_excel()].
#'
#' @export
#' @examples
#' app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
#' cat(gb_to_latex(app, caption = "Worked example"))
gb_to_latex <- function(appraisal, caption = NULL, label = NULL) {
  if (!inherits(appraisal, "gb_appraisal")) {
    cli::cli_abort("{.arg appraisal} must be a {.cls gb_appraisal}.")
  }

  rows <- c(
    sprintf("NPV & %s \\\\", .format_gbp(appraisal$npv)),
    if (!is.null(appraisal$bcr) && !is.na(appraisal$bcr)) {
      sprintf("BCR & %.2f \\\\", appraisal$bcr)
    },
    sprintf("Schedule & %s \\\\", appraisal$schedule),
    sprintf("Horizon & %d years \\\\", length(appraisal$cashflow)),
    if (!is.null(appraisal$base_year)) {
      sprintf("Base year & %s \\\\", appraisal$base_year)
    },
    sprintf("Vintage & Green Book %s \\\\", appraisal$vintage),
    if (!is.null(appraisal$optimism_bias)) {
      sprintf("Optimism bias & %.0f\\%% \\\\", 100 * appraisal$optimism_bias)
    },
    if (isTRUE(appraisal$metb_applied)) {
      "METB & applied \\\\"
    }
  )
  rows <- rows[!vapply(rows, is.null, logical(1))]

  body <- paste(rows, collapse = "\n")
  preamble <- "\\begin{tabular}{ll}\n\\hline\nMetric & Value \\\\\n\\hline\n"
  closing <- "\n\\hline\n\\end{tabular}"

  out <- paste0(preamble, body, closing)
  if (!is.null(caption) || !is.null(label)) {
    cap <- if (!is.null(caption)) sprintf("\\caption{%s}\n", caption) else ""
    lab <- if (!is.null(label)) sprintf("\\label{%s}\n", label) else ""
    out <- sprintf("\\begin{table}[h]\n\\centering\n%s%s%s\n\\end{table}",
                   out, cap, lab)
  }
  out
}

#' Export an appraisal to Excel
#'
#' Writes a multi-sheet workbook: summary, cashflow, present
#' values, provenance.
#'
#' @param appraisal A `gb_appraisal` object.
#' @param file Output file path (must end in `.xlsx`).
#'
#' @return Invisibly, the file path.
#'
#' @details
#' Requires the `openxlsx` package (in Suggests). Install with
#' `install.packages("openxlsx")` if not present.
#'
#' @family reporting
#' @seealso [gb_to_latex()], [gb_to_word()].
#'
#' @export
#' @examples
#' \donttest{
#' if (requireNamespace("openxlsx", quietly = TRUE)) {
#'   app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
#'   tmp <- tempfile(fileext = ".xlsx")
#'   gb_to_excel(app, tmp)
#' }
#' }
gb_to_excel <- function(appraisal, file) {
  if (!requireNamespace("openxlsx", quietly = TRUE)) {
    cli::cli_abort(
      "Package {.pkg openxlsx} is required for {.fn gb_to_excel}. Install with {.code install.packages('openxlsx')}."
    )
  }
  if (!inherits(appraisal, "gb_appraisal")) {
    cli::cli_abort("{.arg appraisal} must be a {.cls gb_appraisal}.")
  }
  if (!grepl("\\.xlsx$", file)) {
    cli::cli_abort("{.arg file} must end in {.val .xlsx}.")
  }

  summary_df <- data.frame(
    metric = c("NPV", "BCR", "Schedule", "Horizon", "Base year",
               "Vintage", "Optimism bias", "METB applied"),
    value = c(
      .format_gbp(appraisal$npv),
      if (!is.null(appraisal$bcr) && !is.na(appraisal$bcr))
        sprintf("%.2f", appraisal$bcr) else NA,
      appraisal$schedule,
      length(appraisal$cashflow),
      appraisal$base_year %||% NA,
      appraisal$vintage,
      if (!is.null(appraisal$optimism_bias))
        sprintf("%.0f%%", 100 * appraisal$optimism_bias) else "none",
      if (isTRUE(appraisal$metb_applied)) "yes" else "no"
    ),
    stringsAsFactors = FALSE
  )

  cashflow_df <- data.frame(
    year = appraisal$years,
    cashflow = appraisal$cashflow,
    discount_factor = gb_discount_factor(appraisal$years, schedule = appraisal$schedule),
    pv = appraisal$pv,
    stringsAsFactors = FALSE
  )

  provenance <- gb_data_versions()

  wb <- openxlsx::createWorkbook()
  openxlsx::addWorksheet(wb, "Summary")
  openxlsx::addWorksheet(wb, "Cashflow")
  openxlsx::addWorksheet(wb, "Provenance")
  openxlsx::writeData(wb, "Summary", summary_df)
  openxlsx::writeData(wb, "Cashflow", cashflow_df)
  openxlsx::writeData(wb, "Provenance", provenance)
  openxlsx::saveWorkbook(wb, file, overwrite = TRUE)

  invisible(file)
}

#' Export an appraisal to Word
#'
#' Writes a one-page Word document with the appraisal headline and
#' cashflow table.
#'
#' @param appraisal A `gb_appraisal` object.
#' @param file Output file path (must end in `.docx`).
#'
#' @return Invisibly, the file path.
#'
#' @details
#' Requires the `officer` and `flextable` packages (both in Suggests).
#'
#' @family reporting
#' @seealso [gb_to_latex()], [gb_to_excel()].
#'
#' @export
#' @examples
#' \donttest{
#' if (requireNamespace("officer", quietly = TRUE) &&
#'     requireNamespace("flextable", quietly = TRUE)) {
#'   app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
#'   tmp <- tempfile(fileext = ".docx")
#'   gb_to_word(app, tmp)
#' }
#' }
gb_to_word <- function(appraisal, file) {
  if (!requireNamespace("officer", quietly = TRUE) ||
      !requireNamespace("flextable", quietly = TRUE)) {
    cli::cli_abort(
      "Packages {.pkg officer} and {.pkg flextable} are required for {.fn gb_to_word}."
    )
  }
  if (!inherits(appraisal, "gb_appraisal")) {
    cli::cli_abort("{.arg appraisal} must be a {.cls gb_appraisal}.")
  }
  if (!grepl("\\.docx$", file)) {
    cli::cli_abort("{.arg file} must end in {.val .docx}.")
  }

  summary_df <- data.frame(
    Metric = c("NPV", "BCR", "Schedule", "Horizon", "Base year", "Vintage"),
    Value = c(
      .format_gbp(appraisal$npv),
      if (!is.null(appraisal$bcr) && !is.na(appraisal$bcr))
        sprintf("%.2f", appraisal$bcr) else "n/a",
      appraisal$schedule,
      sprintf("%d years", length(appraisal$cashflow)),
      as.character(appraisal$base_year %||% "not set"),
      sprintf("Green Book %s", appraisal$vintage)
    ),
    stringsAsFactors = FALSE
  )

  doc <- officer::read_docx()
  doc <- officer::body_add_par(doc, "Green Book Appraisal", style = "heading 1")
  doc <- flextable::body_add_flextable(doc, flextable::flextable(summary_df))
  doc <- officer::body_add_par(doc, "")
  doc <- officer::body_add_par(doc, "Cashflow", style = "heading 2")
  cashflow_df <- data.frame(
    Year = appraisal$years,
    Cashflow = appraisal$cashflow,
    PV = appraisal$pv
  )
  doc <- flextable::body_add_flextable(doc, flextable::flextable(cashflow_df))
  print(doc, target = file)

  invisible(file)
}

test_that("gb_headline returns expected structure", {
  app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
  h <- gb_headline(app)
  expect_s3_class(h, "gb_headline")
  expect_true(all(c("npv", "bcr", "eanc", "payback_year", "horizon") %in%
                  names(h)))
})

test_that("gb_headline payback year is when cumulative PV turns positive", {
  costs <- c(100, 0, 0, 0, 0)
  benefits <- c(0, 30, 30, 30, 30)
  app <- gb_appraise(costs, benefits)
  h <- gb_headline(app)
  expect_true(h$payback_year >= 1)
})

test_that("gb_headline returns NA payback when never positive", {
  app <- gb_appraise(c(100, 0), c(0, 5))
  h <- gb_headline(app)
  expect_true(is.na(h$payback_year))
})

test_that("gb_cost_per_unit divides PV costs by units", {
  app <- gb_appraise(c(100, 0), c(0, 0))
  cu <- gb_cost_per_unit(app, units_delivered = 10, unit = "QALY")
  expect_equal(cu, 10, tolerance = 1e-9)
})

test_that("gb_cost_per_unit errors on zero or negative units", {
  app <- gb_appraise(c(100, 0), c(0, 0))
  expect_error(gb_cost_per_unit(app, units_delivered = 0))
  expect_error(gb_cost_per_unit(app, units_delivered = -5))
})

test_that("gb_to_latex returns a tabular string", {
  app <- gb_appraise(c(100, 0), c(0, 200))
  tex <- gb_to_latex(app)
  expect_type(tex, "character")
  expect_match(tex, "begin\\{tabular\\}")
  expect_match(tex, "NPV")
})

test_that("gb_to_latex with caption wraps in table environment", {
  app <- gb_appraise(c(100, 0), c(0, 200))
  tex <- gb_to_latex(app, caption = "Test", label = "tab:test")
  expect_match(tex, "\\\\begin\\{table\\}")
  expect_match(tex, "Test")
})

test_that("gb_to_excel writes a workbook when openxlsx is available", {
  skip_if_not_installed("openxlsx")
  app <- gb_appraise(c(100, 0), c(0, 200))
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  result <- gb_to_excel(app, tmp)
  expect_true(file.exists(tmp))
  expect_equal(normalizePath(result), normalizePath(tmp))
})

test_that("gb_to_excel errors if file lacks .xlsx extension", {
  skip_if_not_installed("openxlsx")
  app <- gb_appraise(c(100, 0), c(0, 200))
  expect_error(gb_to_excel(app, tempfile(fileext = ".txt")))
})

test_that("gb_to_word writes a docx when officer + flextable available", {
  skip_if_not_installed("officer")
  skip_if_not_installed("flextable")
  app <- gb_appraise(c(100, 0), c(0, 200))
  tmp <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp), add = TRUE)
  result <- gb_to_word(app, tmp)
  expect_true(file.exists(tmp))
})

test_that("gb_headline print method does not error", {
  app <- gb_appraise(c(100, 0), c(0, 200))
  expect_no_error(capture.output(print(gb_headline(app))))
})

#' @export
print.gb_appraisal <- function(x, ...) {
  npv_str <- .format_gbp(x$npv)
  horizon <- length(x$cashflow)
  year_first <- x$years[1]
  year_last <- x$years[length(x$years)]
  cli::cli_h1("Green Book appraisal")
  cli::cli_text("NPV (real, {.field {x$schedule}} schedule): {.val {npv_str}}")
  cli::cli_text("Horizon: {.val {horizon}} years (year {.val {year_first}} to {.val {year_last}})")
  if (!is.null(x$base_year)) {
    cli::cli_text("Base year: {.val {x$base_year}}")
  }
  cli::cli_text("Vintage: Green Book {.val {x$vintage}}")
  invisible(x)
}

#' @export
summary.gb_appraisal <- function(object, ...) {
  cat("Green Book appraisal\n")
  cat("--------------------\n")
  cat(sprintf("NPV          : %s\n", .format_gbp(object$npv)))
  if (!is.null(object$bcr) && !is.na(object$bcr)) {
    cat(sprintf("BCR          : %.2f\n", object$bcr))
  }
  cat(sprintf("Schedule     : %s\n", object$schedule))
  cat(sprintf("Horizon      : %d years (year %s to %s)\n",
              length(object$cashflow),
              object$years[1],
              object$years[length(object$years)]))
  if (!is.null(object$base_year)) {
    cat(sprintf("Base year    : %s\n", object$base_year))
  }
  cat(sprintf("Vintage      : Green Book %s\n", object$vintage))
  if (!is.null(object$pv_costs)) {
    cat(sprintf("PV costs     : %s\n", .format_gbp(object$pv_costs)))
    cat(sprintf("PV benefits  : %s\n", .format_gbp(object$pv_benefits)))
  } else {
    cat(sprintf("Total inflow : %s\n",
                .format_gbp(sum(object$cashflow[object$cashflow > 0]))))
    cat(sprintf("Total outflow: %s\n",
                .format_gbp(sum(object$cashflow[object$cashflow < 0]))))
  }
  if (!is.null(object$optimism_bias)) {
    cat(sprintf("Optimism bias: %.1f percent uplift on costs\n",
                100 * object$optimism_bias))
  }
  if (isTRUE(object$metb_applied)) {
    cat("METB         : applied to costs\n")
  }
  if (!is.null(object$eta)) {
    cat(sprintf("Distrib. eta : %.2f\n", object$eta))
    if (!is.null(object$unweighted_npv)) {
      cat(sprintf("NPV unweighted: %s\n", .format_gbp(object$unweighted_npv)))
    }
  }
  invisible(object)
}

#' @export
format.gb_appraisal <- function(x, ...) {
  sprintf("<gb_appraisal: NPV = %s, %d years, %s>",
          .format_gbp(x$npv),
          length(x$cashflow),
          x$schedule)
}

---
title: "Getting started with greenbook"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with greenbook}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(greenbook)
```

## Why greenbook

`greenbook` provides cost-benefit analysis primitives from the HM Treasury
Green Book. The v0.1.0 release covers the discounting MVP: kinked STPR,
discount factors, NPV, equivalent annual cost, and GDP-deflator rebasing.

All bundled parameter tables carry vintage metadata. Inspect with:

```{r}
gb_data_versions()
```

## The kinked STPR

The Green Book Social Time Preference Rate steps down at 30, 75, 125,
200, and 300 years. `gb_stpr()` returns the rate for any year:

```{r}
gb_stpr(c(10, 30, 31, 75, 76, 200, 300))
```

Three variants are supported:

```{r}
gb_schedule_table()
```

## Discount factors

`gb_discount_factor()` applies the kinked schedule annually using
`prod(1 + r_t)`:

```{r}
gb_discount_factor(0:5)
gb_discount_factor(c(0, 30, 31, 75, 76))
```

The kink is correctly handled across band boundaries:

```{r}
df_30 <- gb_discount_factor(30)
df_31 <- gb_discount_factor(31)
df_31 / df_30  # 1 / 1.030 = 0.9709, the year-31 single-period factor
```

## Net present value

A simple appraisal: capex of GBP 100 in year 0, annual benefits of
GBP 30 from year 1 to year 9.

```{r}
costs <- c(100, rep(0, 9))
benefits <- c(0, rep(30, 9))
appraisal <- gb_npv(benefits - costs, base_year = 2024)
appraisal
```

The result is a `gb_appraisal` object with full provenance. Use
`summary()` for a more detailed view:

```{r}
summary(appraisal)
```

## Equivalent annual net cost

To compare options of different durations, annualise the NPV:

```{r}
gb_eanc(appraisal)
```

## Real-terms rebasing

`gb_real()` converts nominal values at year-of-occurrence prices to
real values at a chosen base year:

```{r}
gb_real(nominal_values = c(100, 110, 120),
        year = 2020:2022,
        base_year = 2024)
```

`gb_rebase()` does the same operation between two real-terms base
years:

```{r}
gb_rebase(c(100, 200, 300), from = 2020, to = 2024)
```

`gb_deflator()` exposes the underlying factor:

```{r}
gb_deflator(from = 2020, to = 2024)
```

## What's next

v0.2.0 adds optimism bias (Mott MacDonald upper bounds), distributional
weights, and Marginal Excess Tax Burden. v0.3.0 ships the valuation
library: WELLBY, VPF, QALY, DESNZ carbon values. See the package
[GitHub repository](https://github.com/charlescoverdale/greenbook) for
the full roadmap.

---
title: "Carbon and environmental valuation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Carbon and environmental valuation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

```{r setup}
library(greenbook)
```

## Carbon values for appraisal

DESNZ publishes a single consolidated carbon value series with
low / central / high scenarios. The November 2023 publication
superseded the historical traded / non-traded split.
`gb_carbon_value()` looks the values up directly:

```{r}
gb_carbon_value(2024)
gb_carbon_value(2024, scenario = "high")
gb_carbon_value(2030)
```

The values are published in 2022 prices. Rebase to a different
year via `base_year`:

```{r}
gb_carbon_value(2030, base_year = 2024)
```

## Net present value of an emissions path

A greenhouse-gas-emitting project has its environmental
externality monetised at the published carbon value, then
discounted under the standard STPR.

```{r}
# 100 tCO2e per year for 2024 to 2030, treated as a cost
emissions <- rep(100, 7)
years <- 2024:2030
gb_carbon_npv(emissions, years, base_year = 2024)
```

For an abatement project (avoided emissions), use
`sign = "benefit"`:

```{r}
gb_carbon_npv(emissions, years, base_year = 2024, sign = "benefit")
```

## Wellbeing valuation

Life-satisfaction changes can be monetised via WELLBYs (HMT
Wellbeing Guidance for Appraisal, July 2021). Default unit value:
GBP 13,000 per WELLBY in 2019 prices.

```{r}
# 1-point lift in life satisfaction, 100 people, 5 years
gb_wellby(1, persons = 100, years = 5)

# In 2024 prices
gb_wellby(1, persons = 100, years = 5, base_year = 2024)

# Sensitivity range
gb_wellby(1, persons = 100, years = 5, scenario = "low")
gb_wellby(1, persons = 100, years = 5, scenario = "high")
```

## Risk to life and health

Value of Preventing a Fatality (VPF) and Quality-Adjusted Life
Year (QALY) per DfT TAG and DHSC supplementary guidance.

```{r}
gb_vpf()        # 2024 published value: GBP 2.153m
gb_vpf(2018)    # 2018 baseline

gb_qaly(1)                          # DHSC: GBP 70k per QALY
gb_qaly(1, scenario = "nice_upper") # NICE upper threshold: GBP 30k
```

## Provenance

```{r}
gb_data_versions()
```

## Provenance

The bundled DESNZ carbon path is the November 2023 publication
(Data Tables 1-19, Table 3), 2020 to 2050. A future refresh will
extend through to 2100.

The VPF series anchors on the DfT TAG data book v2.03 (December
2025) Table A4.1.1 WTP element of GBP 2,474,341 in 2023 prices,
uplifted at 2 percent annual real GDP per head growth per TAG
methodology.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{gb_to_excel}
\alias{gb_to_excel}
\title{Export an appraisal to Excel}
\usage{
gb_to_excel(appraisal, file)
}
\arguments{
\item{appraisal}{A \code{gb_appraisal} object.}

\item{file}{Output file path (must end in \code{.xlsx}).}
}
\value{
Invisibly, the file path.
}
\description{
Writes a multi-sheet workbook: summary, cashflow, present
values, provenance.
}
\details{
Requires the \code{openxlsx} package (in Suggests). Install with
\code{install.packages("openxlsx")} if not present.
}
\examples{
\donttest{
if (requireNamespace("openxlsx", quietly = TRUE)) {
  app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
  tmp <- tempfile(fileext = ".xlsx")
  gb_to_excel(app, tmp)
}
}
}
\seealso{
\code{\link[=gb_to_latex]{gb_to_latex()}}, \code{\link[=gb_to_word]{gb_to_word()}}.

Other reporting: 
\code{\link{gb_cost_per_unit}()},
\code{\link{gb_headline}()},
\code{\link{gb_to_latex}()},
\code{\link{gb_to_word}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/valuation.R
\name{gb_wellby}
\alias{gb_wellby}
\title{Wellbeing valuation in GBP (WELLBY)}
\usage{
gb_wellby(
  life_satisfaction_change,
  persons,
  years = 1,
  base_year = NULL,
  scenario = "central"
)
}
\arguments{
\item{life_satisfaction_change}{Numeric scalar or vector. Change
in life satisfaction on the 0 to 10 scale (signed; can be
positive or negative).}

\item{persons}{Number of people experiencing the change.}

\item{years}{Duration in years. Default \code{1}.}

\item{base_year}{Optional integer base year to express the
monetary value in. If \code{NULL} (default), the published 2019
price is returned. Otherwise the value is uplifted via
\code{gb_deflator()} to \code{base_year} prices.}

\item{scenario}{One of \code{"low"}, \code{"central"} (default), \code{"high"}.}
}
\value{
A numeric scalar or vector: the WELLBY value in GBP at
the requested base year.
}
\description{
Monetises a life-satisfaction change as Well-being Adjusted
Life Years (WELLBYs) per HMT Wellbeing Guidance for Appraisal
(July 2021). One WELLBY equals a one-point change in life
satisfaction on a 0 to 10 scale, for one person, for one year.
The central published unit value is GBP 13,000 in 2019 prices,
with a low-high sensitivity of GBP 10,000 to GBP 16,000.
}
\examples{
# 1-point lift in life satisfaction for 100 people for 5 years
gb_wellby(1, persons = 100, years = 5)
gb_wellby(1, persons = 100, years = 5, scenario = "low")
# Express in 2024 prices
gb_wellby(1, persons = 100, years = 5, base_year = 2024)
}
\references{
HM Treasury (2021). Wellbeing Guidance for Appraisal:
Supplementary Green Book Guidance.
}
\seealso{
\code{\link[=gb_vpf]{gb_vpf()}}, \code{\link[=gb_qaly]{gb_qaly()}}.

Other valuation: 
\code{\link{gb_qaly}()},
\code{\link{gb_vpf}()}
}
\concept{valuation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/greenbook-package.R
\docType{package}
\name{greenbook-package}
\alias{greenbook}
\alias{greenbook-package}
\title{greenbook: HM Treasury Green Book Cost-Benefit Analysis Primitives}
\description{
Implements cost-benefit analysis primitives from HM Treasury Green Book
guidance: the kinked Social Time Preference Rate (STPR), discount factors,
net present value, equivalent annual cost, and GDP-deflator rebasing.
Pure computation, no network access. Bundled parameter tables carry
vintage metadata for reproducibility.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/charlescoverdale/greenbook}
  \item Report bugs at \url{https://github.com/charlescoverdale/greenbook/issues}
}

}
\author{
\strong{Maintainer}: Charles Coverdale \email{charles.f.coverdale@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/place_based.R
\name{gb_place_based}
\alias{gb_place_based}
\title{Aggregate sub-projects into a place-based business case}
\usage{
gb_place_based(..., place = NULL, synergy = 0)
}
\arguments{
\item{...}{\code{gb_appraisal} objects representing sub-projects.
Pass as named arguments to label each.}

\item{place}{Character scalar. Place name (e.g. \code{"Greater Manchester"}). Optional.}

\item{synergy}{Numeric scalar in \verb{[-0.5, 0.5]}. Optional uplift
applied to aggregate benefits to capture cross-project synergy
or drag. Default \code{0} (additive aggregation).}
}
\value{
A \code{gb_place_based} object with elements \code{projects},
\code{place}, \code{aggregate_npv}, \code{aggregate_bcr}, \code{per_project},
\code{synergy}.
}
\description{
Combines multiple \code{gb_appraisal} objects into a single portfolio
appraisal. Introduced in the 2026 Green Book as a recognised
business case structure for places (city deals, devolved
settlements, regional packages) where individual projects are
interdependent.
}
\examples{
transport <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
housing <- gb_appraise(c(50, 0, 0, 0, 0), c(0, 20, 20, 20, 20))
skills <- gb_appraise(c(20, 20, 0, 0, 0), c(0, 0, 15, 15, 15))
gb_place_based(transport = transport, housing = housing,
               skills = skills, place = "Example City")
}
\references{
HM Treasury (2026). The Green Book, on place-based
business cases.
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}, \code{\link[=gb_compare]{gb_compare()}}.

Other appraisal: 
\code{\link{gb_appraise}()},
\code{\link{gb_compare}()},
\code{\link{gb_economic_case}()},
\code{\link{gb_progression}()},
\code{\link{gb_risk_register}()},
\code{\link{gb_sensitivity_ob}()},
\code{\link{gb_validate}()}
}
\concept{appraisal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discounting.R
\name{gb_discount_factor}
\alias{gb_discount_factor}
\title{Discount factor under the kinked STPR}
\usage{
gb_discount_factor(years, schedule = "standard", base_year = 0L)
}
\arguments{
\item{years}{Integer vector of years from the base year. Must be
\verb{>= base_year}.}

\item{schedule}{One of \code{"standard"}, \code{"health"}, \code{"catastrophic"}.}

\item{base_year}{Integer base year offset. Default \code{0}.}
}
\value{
A numeric vector of discount factors. \code{1} for \code{years == base_year}.
}
\description{
Returns the present-value discount factor for each year, applying
the kinked Social Time Preference Rate schedule annually.
}
\details{
The discount factor at year \code{t} is computed as the reciprocal of
the cumulative product of \code{(1 + r_k)} for periods \verb{k = 1, ..., t},
where \code{r_k} is the STPR for year \code{k}. This handles the kinked
schedule correctly across band transitions (e.g. year 30 to year
31). Within a single band, the closed-form annuity factor
\code{(1 - (1 + r)^(-n)) / r} agrees with \code{sum(gb_discount_factor(1:n))}
to machine precision.
}
\examples{
gb_discount_factor(0:5)
gb_discount_factor(c(0, 30, 31, 75, 76))
}
\references{
HM Treasury (2022). The Green Book, Annex A6.

HM Treasury (2003). Green Book Supplementary Guidance: Discounting.
}
\seealso{
\code{\link[=gb_stpr]{gb_stpr()}}, \code{\link[=gb_npv]{gb_npv()}}, \code{\link[=gb_eanc]{gb_eanc()}}.

Other discounting: 
\code{\link{gb_discount}()},
\code{\link{gb_eanc}()},
\code{\link{gb_npv}()},
\code{\link{gb_stpr}()}
}
\concept{discounting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/economic_case.R
\name{gb_economic_case}
\alias{gb_economic_case}
\title{Render an appraisal as a Five Case Model economic case}
\usage{
gb_economic_case(
  appraisal,
  critical_success_factors = NULL,
  options_considered = NULL,
  non_monetised_impacts = NULL,
  recommendation = NULL,
  vfm_statement = NULL
)
}
\arguments{
\item{appraisal}{A \code{gb_appraisal} (typically from \code{gb_appraise()}
or \code{gb_compare()}).}

\item{critical_success_factors}{Character vector of CSFs.}

\item{options_considered}{Character vector of long-listed option
names.}

\item{non_monetised_impacts}{Optional data frame with columns
\code{impact}, \code{direction} (\code{"+"}/\code{"-"}), \code{materiality}
(\code{"H"}/\code{"M"}/\code{"L"}), \code{notes}.}

\item{recommendation}{Optional character: the preferred option
and rationale.}

\item{vfm_statement}{Optional character: the value-for-money
judgment.}
}
\value{
A \code{gb_economic_case} object.
}
\description{
Wraps a \code{gb_appraisal} with the structural sections HMT business
case guidance expects in the Economic Case (the second of the
five cases in the Five Case Model: Strategic, Economic,
Commercial, Financial, Management).
}
\details{
The Five Case Model is HM Treasury's standard structure for
business cases. The Economic Case is the part where Green Book
appraisal sits: monetised costs and benefits, non-monetised
impacts, switching values, sensitivity tests, value for money
judgment, recommended option. \code{gb_economic_case} wraps the
appraisal with the sections a reviewer expects to see.
}
\examples{
app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
gb_economic_case(
  app,
  critical_success_factors = c("Strategic fit", "Value for money", "Achievability"),
  options_considered = c("Do nothing", "Do minimum", "Do maximum"),
  recommendation = "Do maximum: positive NPV and BCR > 1.5"
)
}
\references{
HM Treasury (2018). Guide to Developing the Project
Business Case (Five Case Model).
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}, \code{\link[=gb_compare]{gb_compare()}}.

Other appraisal: 
\code{\link{gb_appraise}()},
\code{\link{gb_compare}()},
\code{\link{gb_place_based}()},
\code{\link{gb_progression}()},
\code{\link{gb_risk_register}()},
\code{\link{gb_sensitivity_ob}()},
\code{\link{gb_validate}()}
}
\concept{appraisal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discounting.R
\name{gb_npv}
\alias{gb_npv}
\title{Net present value}
\usage{
gb_npv(
  cashflow,
  years = NULL,
  schedule = "standard",
  base_year = NULL,
  vintage = "2022"
)
}
\arguments{
\item{cashflow}{Numeric vector of net cashflows in real terms (one
value per year).}

\item{years}{Integer vector of years matching \code{cashflow}. Defaults
to \verb{0, 1, ..., length(cashflow) - 1}.}

\item{schedule}{One of \code{"standard"}, \code{"health"}, \code{"catastrophic"}.}

\item{base_year}{Optional integer recording the price base year for
the cashflow (e.g. \code{2024}). Stored on the returned object.}

\item{vintage}{Character. Methodology vintage label. Defaults to
\code{"2022"}.}
}
\value{
A \code{gb_appraisal} object: a list with class
\code{c("gb_appraisal", "list")} and elements
\code{npv}, \code{cashflow}, \code{years}, \code{pv}, \code{schedule}, \code{base_year}, \code{vintage}.
}
\description{
Computes the net present value of a cashflow stream under the
Green Book STPR. Returns a \code{gb_appraisal} object carrying the NPV,
the input cashflow, the discount factors, the schedule used, and
methodology vintage.
}
\examples{
costs    <- c(100, 50, 50, 0, 0, 0, 0, 0, 0, 0)
benefits <- c(0, 0, 0, 30, 30, 30, 30, 30, 30, 30)
gb_npv(benefits - costs)
}
\references{
HM Treasury (2022). The Green Book, chapter on appraisal and the
Annex on discounting.
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}, \code{\link[=gb_eanc]{gb_eanc()}}, \code{\link[=gb_dist_weighted_npv]{gb_dist_weighted_npv()}},
\code{\link[=gb_carbon_npv]{gb_carbon_npv()}}.

Other discounting: 
\code{\link{gb_discount}()},
\code{\link{gb_discount_factor}()},
\code{\link{gb_eanc}()},
\code{\link{gb_stpr}()}
}
\concept{discounting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lookups.R
\name{gb_data_versions}
\alias{gb_data_versions}
\title{Vintage of bundled parameter tables}
\usage{
gb_data_versions()
}
\value{
A data frame with columns \code{dataset}, \code{source},
\code{last_updated}, \code{notes}.
}
\description{
Returns a data frame describing the source and last-updated date
of every CSV bundled in \verb{inst/extdata/}. Critical for
reproducibility: every appraisal can record which vintage of
Green Book parameters it used.
}
\examples{
gb_data_versions()
}
\seealso{
\code{\link[=gb_schedule_table]{gb_schedule_table()}}.

Other lookups: 
\code{\link{gb_schedule_table}()}
}
\concept{lookups}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{gb_to_latex}
\alias{gb_to_latex}
\title{Render an appraisal as a LaTeX table}
\usage{
gb_to_latex(appraisal, caption = NULL, label = NULL)
}
\arguments{
\item{appraisal}{A \code{gb_appraisal} object.}

\item{caption}{Optional table caption.}

\item{label}{Optional LaTeX label for cross-referencing.}
}
\value{
A character scalar containing a LaTeX \code{tabular}
environment. Wrap in \verb{\\\\begin\{table\}...\\\\end\{table\}} for a
floating table.
}
\description{
Render an appraisal as a LaTeX table
}
\examples{
app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
cat(gb_to_latex(app, caption = "Worked example"))
}
\seealso{
\code{\link[=gb_to_excel]{gb_to_excel()}}.

Other reporting: 
\code{\link{gb_cost_per_unit}()},
\code{\link{gb_headline}()},
\code{\link{gb_to_excel}()},
\code{\link{gb_to_word}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/carbon.R
\name{gb_carbon_value}
\alias{gb_carbon_value}
\title{DESNZ carbon value for appraisal}
\usage{
gb_carbon_value(year, scenario = "central", base_year = NULL)
}
\arguments{
\item{year}{Integer scalar or vector. Year of the emission. Must
be within the bundled DESNZ range.}

\item{scenario}{One of \code{"low"}, \code{"central"} (default), \code{"high"}.}

\item{base_year}{Optional integer to rebase the value via
\code{gb_deflator()}. If \code{NULL}, the published 2022 base year is
used.}
}
\value{
Numeric vector of GBP per tCO2e values.
}
\description{
Returns the DESNZ Carbon Value for Appraisal in GBP per tonne of
CO2-equivalent at the published 2022 base-year prices, from the
November 2023 valuation. Three scenarios: low, central, high.
}
\details{
DESNZ moved to a single consolidated series in November 2023,
superseding the historical traded / non-traded split. Values
are in 2022 prices. The November 2023 publication covers 2010
to 2100; the bundled subset is 2020 to 2050.
}
\examples{
gb_carbon_value(2024)
gb_carbon_value(2020:2030)
gb_carbon_value(2030, scenario = "high")
}
\references{
DESNZ (2023). Valuation of Energy Use and Greenhouse
Gas Emissions for Appraisal (November 2023). Data tables 1-19,
Table 3.
}
\seealso{
\code{\link[=gb_carbon_npv]{gb_carbon_npv()}}.

Other carbon: 
\code{\link{gb_carbon_npv}()}
}
\concept{carbon}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/valuation.R
\name{gb_qaly}
\alias{gb_qaly}
\title{Value of a Quality-Adjusted Life Year}
\usage{
gb_qaly(qalys, scenario = "dhsc", base_year = NULL)
}
\arguments{
\item{qalys}{Numeric scalar or vector. Quality-Adjusted Life
Years gained or lost.}

\item{scenario}{One of \code{"dhsc"} (default, GBP 70k 2024 prices,
cross-government), \code{"nice_lower"} (GBP 20k), \code{"nice_upper"}
(GBP 30k).}

\item{base_year}{Optional integer base year. If \code{NULL}, the
published base year for the chosen scenario is used.}
}
\value{
A numeric scalar or vector: monetised value in GBP.
}
\description{
Returns the QALY value in GBP at the published base year. The
DHSC supplementary Green Book guidance specifies GBP 70,000 per
QALY in 2024 prices for cross-government appraisal. NICE Health
Technology Assessment uses lower thresholds (GBP 20,000 to
GBP 30,000 per QALY).
}
\examples{
gb_qaly(1)
gb_qaly(1, scenario = "nice_upper")
}
\references{
DHSC Supplementary Green Book Guidance on health
appraisal. NICE methods guides for technology appraisal.
}
\seealso{
\code{\link[=gb_wellby]{gb_wellby()}}, \code{\link[=gb_vpf]{gb_vpf()}}.

Other valuation: 
\code{\link{gb_vpf}()},
\code{\link{gb_wellby}()}
}
\concept{valuation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/appraise.R
\name{gb_appraise}
\alias{gb_appraise}
\title{Full Green Book appraisal in one call}
\usage{
gb_appraise(
  costs,
  benefits,
  years = NULL,
  schedule = "standard",
  ob = NULL,
  ob_dimension = "capex",
  ob_mitigation = 0,
  metb = FALSE,
  metb_rate = 0.2,
  base_year = NULL,
  vintage = "2022"
)
}
\arguments{
\item{costs}{Numeric vector of cost values per period (real
terms, base year fixed). Sign convention: enter as positive
numbers; \code{gb_appraise()} handles the netting.}

\item{benefits}{Numeric vector of benefit values per period.
Same length as \code{costs}.}

\item{years}{Optional integer vector of years. Defaults to
\code{0:(length(costs) - 1)}.}

\item{schedule}{One of \code{"standard"}, \code{"health"}, \code{"catastrophic"}.}

\item{ob}{Optional. Either a category name (character) or a
numeric percentage. If supplied, optimism bias uplift is
applied to \code{costs}.}

\item{ob_dimension}{One of \code{"capex"} (default) or \code{"duration"}.
Only used when \code{ob} is a category name.}

\item{ob_mitigation}{Numeric in \verb{[0, 1]}. Mitigation fraction.}

\item{metb}{Logical. If \code{TRUE}, applies METB uplift to costs.}

\item{metb_rate}{Numeric. METB rate when \code{metb = TRUE}. Default
\code{0.20} per Green Book 2022.}

\item{base_year}{Optional integer base year.}

\item{vintage}{Methodology vintage label. Default \code{"2022"}.}
}
\value{
A \code{gb_appraisal} object with extra fields \code{bcr},
\code{pv_costs}, \code{pv_benefits}, \code{costs}, \code{benefits},
\code{optimism_bias}, \code{metb_applied}.
}
\description{
Runs an end-to-end appraisal: optionally applies optimism bias to
costs, optionally applies METB, computes NPV under the kinked
STPR, and returns BCR alongside provenance.
}
\examples{
costs <- c(100, 50, 50, 0, 0, 0, 0, 0, 0, 0)
benefits <- c(0, 0, 0, 30, 30, 30, 30, 30, 30, 30)
app <- gb_appraise(costs, benefits, ob = "non_standard_buildings",
                   ob_mitigation = 0.5, base_year = 2024)
app
}
\references{
HM Treasury (2026). The Green Book: Central Government Guidance
on Appraisal and Evaluation.
}
\seealso{
\code{\link[=gb_npv]{gb_npv()}}, \code{\link[=gb_optimism_bias]{gb_optimism_bias()}}, \code{\link[=gb_metb]{gb_metb()}},
\code{\link[=gb_dist_weighted_npv]{gb_dist_weighted_npv()}}.

Other appraisal: 
\code{\link{gb_compare}()},
\code{\link{gb_economic_case}()},
\code{\link{gb_place_based}()},
\code{\link{gb_progression}()},
\code{\link{gb_risk_register}()},
\code{\link{gb_sensitivity_ob}()},
\code{\link{gb_validate}()}
}
\concept{appraisal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deflator.R
\name{gb_rebase}
\alias{gb_rebase}
\title{Rebase a real-terms series to a different base year}
\usage{
gb_rebase(values, from, to)
}
\arguments{
\item{values}{Numeric vector of real-terms values.}

\item{from}{Integer base year of the input series.}

\item{to}{Integer target base year.}
}
\value{
A numeric vector of values in \code{to}-year real prices.
}
\description{
Multiplies values currently in \code{from}-year real prices by the
deflator factor to express them in \code{to}-year real prices.
}
\examples{
gb_rebase(c(100, 200, 300), from = 2020, to = 2024)
}
\references{
HM Treasury (2022). The Green Book, chapter on real
terms and price-base alignment.
}
\seealso{
\code{\link[=gb_deflator]{gb_deflator()}}, \code{\link[=gb_real]{gb_real()}}.

Other deflator: 
\code{\link{gb_deflator}()},
\code{\link{gb_real}()}
}
\concept{deflator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discounting.R
\name{gb_discount}
\alias{gb_discount}
\title{Apply discount factors to a stream}
\usage{
gb_discount(values, years = seq_along(values) - 1L, schedule = "standard")
}
\arguments{
\item{values}{Numeric vector of cashflow values (real terms, base
year fixed).}

\item{years}{Integer vector of years from the base year. Defaults
to \verb{0, 1, 2, ...}.}

\item{schedule}{One of \code{"standard"}, \code{"health"}, \code{"catastrophic"}.}
}
\value{
A numeric vector of discounted (present-value) cashflows.
}
\description{
Apply discount factors to a stream
}
\examples{
gb_discount(c(0, 100, 100, 100))
}
\references{
HM Treasury (2022). The Green Book, Annex A6.
}
\seealso{
\code{\link[=gb_discount_factor]{gb_discount_factor()}}, \code{\link[=gb_npv]{gb_npv()}}.

Other discounting: 
\code{\link{gb_discount_factor}()},
\code{\link{gb_eanc}()},
\code{\link{gb_npv}()},
\code{\link{gb_stpr}()}
}
\concept{discounting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/carbon.R
\name{gb_carbon_npv}
\alias{gb_carbon_npv}
\title{Net present value of an emissions path}
\usage{
gb_carbon_npv(
  emissions,
  years,
  scenario = "central",
  schedule = "standard",
  base_year = NULL,
  sign = "cost"
)
}
\arguments{
\item{emissions}{Numeric vector of emissions per year, in tCO2e
(positive = emitted, negative = avoided / abated).}

\item{years}{Integer vector of years matching \code{emissions}.}

\item{scenario}{One of \code{"low"}, \code{"central"} (default), \code{"high"}.}

\item{schedule}{One of \code{"standard"} (default), \code{"health"},
\code{"catastrophic"}.}

\item{base_year}{Optional integer base year for monetary values
(e.g. \code{2024}). If supplied, carbon values are rebased via
\code{gb_deflator()}.}

\item{sign}{Character. \code{"cost"} (default) treats positive
\code{emissions} as a cost (negative cashflow); \code{"benefit"} treats
positive as avoided emissions (positive cashflow).}
}
\value{
A \code{gb_appraisal} object.
}
\description{
Multiplies an emissions vector (tCO2e per year) by the DESNZ
carbon value at each year, then discounts under the kinked STPR.
Returns a \code{gb_appraisal} object.
}
\examples{
# 100 tCO2e emitted each year from 2024 to 2030
emissions <- rep(100, 7)
years <- 2024:2030
gb_carbon_npv(emissions, years, base_year = 2024)
}
\references{
DESNZ (2023). Valuation of Energy Use and Greenhouse
Gas Emissions for Appraisal (November 2023). HM Treasury (2022).
The Green Book, on environmental valuation.
}
\seealso{
\code{\link[=gb_carbon_value]{gb_carbon_value()}}, \code{\link[=gb_npv]{gb_npv()}}.

Other carbon: 
\code{\link{gb_carbon_value}()}
}
\concept{carbon}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{gb_headline}
\alias{gb_headline}
\title{One-page headline summary of an appraisal}
\usage{
gb_headline(appraisal)
}
\arguments{
\item{appraisal}{A \code{gb_appraisal} object.}
}
\value{
A \code{gb_headline} object with elements \code{npv}, \code{bcr},
\code{eanc}, \code{payback_year}, \code{optimism_bias}, \code{metb_applied},
\code{vintage}, \code{base_year}, \code{horizon}.
}
\description{
Returns the headline numbers a Green Book reviewer or steering
committee expects: NPV, BCR, EANC, payback period, optimism bias
applied, vintage. Suitable for a slide or executive summary.
}
\examples{
app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
gb_headline(app)
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}, \code{\link[=gb_to_latex]{gb_to_latex()}}, \code{\link[=gb_to_excel]{gb_to_excel()}}.

Other reporting: 
\code{\link{gb_cost_per_unit}()},
\code{\link{gb_to_excel}()},
\code{\link{gb_to_latex}()},
\code{\link{gb_to_word}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjustments.R
\name{gb_metb}
\alias{gb_metb}
\title{Apply Marginal Excess Tax Burden to public expenditure}
\usage{
gb_metb(values, rate = 0.2, vintage = NULL)
}
\arguments{
\item{values}{Numeric vector of expenditure values.}

\item{rate}{Numeric scalar. METB rate as a decimal. Default
\code{0.20}.}

\item{vintage}{Optional character. One of \code{"2003"}, \code{"2018"},
\code{"2022"}, \code{"2026"}. If supplied, overrides \code{rate} with the
bundled value for that vintage.}
}
\value{
A numeric vector the same length as \code{values}, with the
METB uplift applied.
}
\description{
Uplifts net public expenditure (or revenue raised) to reflect
the welfare cost of distortionary taxation. Default rate is
20 percent per Green Book 2022 / 2026. The historic value (2003)
was 30 percent.
}
\details{
The METB captures the welfare loss arising from raising one extra
GBP of revenue through the tax system, beyond the GBP itself.
Estimates depend on the elasticity of taxable income, the marginal
tax rate, and the distortionary margin. The Green Book reduced the
headline value from 30 percent (2003) to 20 percent (2018) in
light of post-2008 evidence.
}
\examples{
gb_metb(c(100, 200))
gb_metb(c(100, 200), vintage = "2003")
}
\references{
HM Treasury (2022). The Green Book: Central Government
Guidance on Appraisal and Evaluation, chapter on real terms and
the marginal excess tax burden.
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}.
}
\concept{adjustments}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deflator.R
\name{gb_deflator}
\alias{gb_deflator}
\title{GDP deflator factor between two years}
\usage{
gb_deflator(from, to, source = "bundled")
}
\arguments{
\item{from}{Integer year of the input value.}

\item{to}{Integer year to express the value in.}

\item{source}{Character. \code{"bundled"} uses the CSV shipped in
\verb{inst/extdata/}. Future versions will accept \code{"inflateR"} to use
the live ONS series via the \code{inflateR} package.}
}
\value{
A numeric scalar: the deflator factor.
}
\description{
Returns the multiplicative factor to convert a value expressed in
year \code{from} prices to year \code{to} prices, using the bundled UK GDP
deflator at market prices (ONS series). Multiply a nominal value
from year \code{from} by this factor to express it in year \code{to}.
}
\examples{
gb_deflator(from = 2020, to = 2024)
}
\references{
Office for National Statistics. GDP deflator at market
prices (series YBGB), updated quarterly. HM Treasury publishes
the deflator series alongside the OBR forecast.
}
\seealso{
\code{\link[=gb_real]{gb_real()}}, \code{\link[=gb_rebase]{gb_rebase()}}.

Other deflator: 
\code{\link{gb_real}()},
\code{\link{gb_rebase}()}
}
\concept{deflator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimism_bias.R
\name{gb_categories}
\alias{gb_categories}
\title{Available optimism bias categories}
\usage{
gb_categories()
}
\value{
A data frame with columns \code{category}, \code{description},
\code{capex_upper}, \code{duration_upper}.
}
\description{
Returns the bundled OB category lookup as a data frame.
}
\examples{
gb_categories()
}
\references{
HM Treasury (2003). Supplementary Green Book Guidance:
Optimism Bias.
}
\seealso{
\code{\link[=gb_optimism_bias]{gb_optimism_bias()}}.

Other optimism bias: 
\code{\link{gb_apply_ob}()},
\code{\link{gb_optimism_bias}()}
}
\concept{optimism bias}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributional.R
\name{gb_dist_weighted_npv}
\alias{gb_dist_weighted_npv}
\title{Distributionally-weighted net present value}
\usage{
gb_dist_weighted_npv(
  cashflow,
  recipient_income,
  eta = 1.3,
  schedule = "standard",
  reference = "median",
  income_data = NULL,
  vintage = "2022",
  base_year = NULL
)
}
\arguments{
\item{cashflow}{Numeric vector of net cashflows (per period).}

\item{recipient_income}{Numeric vector. The income of the
recipient (or representative recipient cell) in each period.
Must have the same length as \code{cashflow}.}

\item{eta}{Numeric scalar. Default \code{1.3}.}

\item{schedule}{One of \code{"standard"}, \code{"health"}, \code{"catastrophic"}.}

\item{reference}{Reference income strategy passed to
\code{gb_dist_weight()}.}

\item{income_data}{Optional reference income distribution.}

\item{vintage}{Methodology vintage label. Default \code{"2022"}.}

\item{base_year}{Optional integer base year stored on the result.}
}
\value{
A \code{gb_appraisal} object with extra fields \code{weights},
\code{eta}, and \code{unweighted_npv}.
}
\description{
Applies recipient-income distributional weights to a cashflow
before discounting under the STPR.
}
\examples{
# 5-year benefit stream of GBP 30 going to a low-decile recipient
gb_dist_weighted_npv(
  cashflow = rep(30, 5),
  recipient_income = rep(15000, 5),
  income_data = seq(10000, 100000, length.out = 10)
)
}
\references{
HM Treasury (2022). The Green Book, Annex A3 on
distributional analysis.
}
\seealso{
\code{\link[=gb_dist_weight]{gb_dist_weight()}}, \code{\link[=gb_npv]{gb_npv()}}.

Other distributional: 
\code{\link{gb_dist_weight}()}
}
\concept{distributional}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sensitivity.R
\name{gb_sensitivity_ob}
\alias{gb_sensitivity_ob}
\title{Optimism bias sensitivity sweep}
\usage{
gb_sensitivity_ob(
  costs,
  benefits,
  ob,
  mitigations = c(0, 0.25, 0.5, 0.75, 1),
  ...
)
}
\arguments{
\item{costs}{Numeric vector of cost values.}

\item{benefits}{Numeric vector of benefit values.}

\item{ob}{Either a category name or a numeric OB percentage.}

\item{mitigations}{Numeric vector in \verb{[0, 1]}. Mitigation
fractions to test. Default \code{c(0, 0.25, 0.5, 0.75, 1.0)}.}

\item{...}{Passed to \code{gb_appraise()}.}
}
\value{
A \code{gb_sensitivity_ob} object: a list with elements
\code{mitigations}, \code{npv}, \code{bcr}, \code{costs_pv}.
}
\description{
Recomputes an appraisal across a vector of optimism bias
mitigation factors. Convenience wrapper for the standard HMT
OB sensitivity test required at SOC, OBC, and FBC stages.
}
\examples{
costs <- c(100, 50, 50, 0, 0, 0, 0, 0, 0, 0)
benefits <- c(0, 0, 0, 30, 30, 30, 30, 30, 30, 30)
gb_sensitivity_ob(costs, benefits, ob = "non_standard_buildings")
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}, \code{\link[=gb_apply_ob]{gb_apply_ob()}}.

Other appraisal: 
\code{\link{gb_appraise}()},
\code{\link{gb_compare}()},
\code{\link{gb_economic_case}()},
\code{\link{gb_place_based}()},
\code{\link{gb_progression}()},
\code{\link{gb_risk_register}()},
\code{\link{gb_validate}()}
}
\concept{appraisal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributional.R
\name{gb_dist_weight}
\alias{gb_dist_weight}
\title{Distributional weight for a recipient income}
\usage{
gb_dist_weight(income, eta = 1.3, reference = "median", income_data = NULL)
}
\arguments{
\item{income}{Numeric vector of recipient incomes (positive).
Equivalised household disposable income is the conventional
measure.}

\item{eta}{Numeric scalar. Income elasticity of marginal utility
of income. Default \code{1.3}.}

\item{reference}{Either \code{"median"} (default; use median of
\code{income} or \code{income_data} if supplied) or a numeric scalar
(use that income as reference).}

\item{income_data}{Optional numeric vector. Reference income
distribution for computing the median. Use, e.g., the ONS
household disposable income distribution. If \code{NULL} (default),
the median of \code{income} is used.}
}
\value{
A numeric vector of weights, same length as \code{income}.
}
\description{
Returns the Green Book distributional weight applied to net
benefits accruing to a recipient at income \code{income}, relative to
a reference income (median by default), under iso-elastic utility:
}
\details{
\deqn{w_i = (y_{ref} / y_i) ^ \eta}

Default \code{eta = 1.3} per HM Treasury Green Book Annex A3. Higher
eta places greater weight on lower-income recipients; sensitivity
tests at \code{eta = 0.8} and \code{eta = 2.0} are conventional.
}
\examples{
# Weights across deciles of a stylised income distribution
income_deciles <- seq(10000, 100000, length.out = 10)
gb_dist_weight(income_deciles)
}
\references{
HM Treasury (2022). The Green Book, Annex A3 on distributional
analysis.

Acland, D. and Greenberg, D.H. (2024). The Elasticity of Marginal
Utility of Income for Distributional Weighting and Social
Discounting: A Meta-Analysis. Journal of Benefit-Cost Analysis.
}
\seealso{
\code{\link[=gb_dist_weighted_npv]{gb_dist_weighted_npv()}}.

Other distributional: 
\code{\link{gb_dist_weighted_npv}()}
}
\concept{distributional}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{gb_compare}
\alias{gb_compare}
\title{Compare multiple appraisal options}
\usage{
gb_compare(..., by = "npv")
}
\arguments{
\item{...}{\code{gb_appraisal} objects to compare. Pass as named
arguments to control labels (e.g. \code{do_minimum = app1}).}

\item{by}{Character. Ranking criterion: one of \code{"npv"} (default)
or \code{"bcr"}.}
}
\value{
A \code{gb_comparison} object: a list with class
\code{c("gb_comparison", "list")} and elements \code{options},
\code{summary_table}, \code{preferred_npv}, \code{preferred_bcr}, \code{by}.
}
\description{
Builds a side-by-side comparison of two or more \code{gb_appraisal}
objects. Returns a \code{gb_comparison} with a ranked summary table
and the preferred option under both NPV and BCR criteria.
}
\details{
Every Green Book economic case must compare at least Do Minimum
against one or more Do Something options. \code{gb_compare()}
standardises the comparison: NPV, BCR, EANC, PV costs, PV benefits,
and rank under both NPV and BCR. The \code{summary()} method renders a
one-page table suitable for a Five Case Model economic case.
}
\examples{
do_minimum <- gb_appraise(c(50, 0, 0, 0, 0), c(0, 20, 20, 20, 20))
do_max <- gb_appraise(c(150, 0, 0, 0, 0), c(0, 50, 50, 50, 50))
gb_compare(do_minimum = do_minimum, do_max = do_max)
}
\references{
HM Treasury (2022). The Green Book: Central Government Guidance
on Appraisal and Evaluation, chapter on options analysis.
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}, \code{\link[=gb_economic_case]{gb_economic_case()}}.

Other appraisal: 
\code{\link{gb_appraise}()},
\code{\link{gb_economic_case}()},
\code{\link{gb_place_based}()},
\code{\link{gb_progression}()},
\code{\link{gb_risk_register}()},
\code{\link{gb_sensitivity_ob}()},
\code{\link{gb_validate}()}
}
\concept{appraisal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discounting.R
\name{gb_eanc}
\alias{gb_eanc}
\title{Equivalent annual net cost (or benefit)}
\usage{
gb_eanc(npv, years, schedule = "standard")
}
\arguments{
\item{npv}{Either a numeric scalar NPV, or a \code{gb_appraisal} object.}

\item{years}{Project horizon in years. If \code{npv} is a \code{gb_appraisal}
and \code{years} is missing, \code{length(npv$cashflow) - 1L} is used.}

\item{schedule}{One of \code{"standard"}, \code{"health"}, \code{"catastrophic"}.}
}
\value{
A numeric scalar: the equivalent annual amount in real
terms, base year aligned with the input.
}
\description{
Converts a present value to an annualised equivalent over a fixed
horizon under the STPR. Used to compare options of different
durations.
}
\examples{
app <- gb_npv(c(-100, 30, 30, 30, 30, 30))
gb_eanc(app)
}
\references{
HM Treasury (2022). The Green Book, Annex A on
appraisal: equivalent annual cost.
}
\seealso{
\code{\link[=gb_npv]{gb_npv()}}, \code{\link[=gb_appraise]{gb_appraise()}}.

Other discounting: 
\code{\link{gb_discount}()},
\code{\link{gb_discount_factor}()},
\code{\link{gb_npv}()},
\code{\link{gb_stpr}()}
}
\concept{discounting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/risk.R
\name{gb_risk_register}
\alias{gb_risk_register}
\title{Build a risk register with monetised exposure}
\usage{
gb_risk_register(risks, appraisal = NULL)
}
\arguments{
\item{risks}{A data frame with columns \code{id}, \code{description},
\code{probability} (numeric in \verb{[0, 1]}), \code{impact_gbp} (numeric).
Optional columns: \code{category}, \code{mitigation} (character).}

\item{appraisal}{Optional \code{gb_appraisal}. If supplied, returns a
risk-adjusted NPV: \code{appraisal$npv - sum(probability * impact)}.}
}
\value{
A \code{gb_risk_register} object.
}
\description{
Creates a \code{gb_risk_register} from a data frame of project risks.
Computes expected loss per risk (probability times impact) and
aggregate exposure. Optionally applied to a \code{gb_appraisal} to
produce a risk-adjusted NPV.
}
\details{
HM Treasury business case guidance requires a risk register with
monetised exposure for the OBC and FBC stages. \code{gb_risk_register}
standardises the structure: every risk has an id, a probability,
a monetary impact, and an expected loss. Aggregation is by category
if the column is present.
}
\examples{
risks <- data.frame(
  id = c("R1", "R2", "R3"),
  description = c("Planning delay", "Cost overrun", "Lower demand"),
  category = c("schedule", "cost", "demand"),
  probability = c(0.30, 0.50, 0.20),
  impact_gbp = c(2e6, 5e6, 10e6)
)
gb_risk_register(risks)
}
\references{
HM Treasury (2018). The Orange Book: Management of
Risk - Principles and Concepts.
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}.

Other appraisal: 
\code{\link{gb_appraise}()},
\code{\link{gb_compare}()},
\code{\link{gb_economic_case}()},
\code{\link{gb_place_based}()},
\code{\link{gb_progression}()},
\code{\link{gb_sensitivity_ob}()},
\code{\link{gb_validate}()}
}
\concept{appraisal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deflator.R
\name{gb_real}
\alias{gb_real}
\title{Convert nominal values to real}
\usage{
gb_real(nominal_values, year, base_year, source = "bundled")
}
\arguments{
\item{nominal_values}{Numeric vector of nominal values.}

\item{year}{Integer scalar or vector matching \code{nominal_values},
giving the year at which each value is expressed in nominal terms.}

\item{base_year}{Integer scalar: the base year to convert to.}

\item{source}{Character. \code{"bundled"} only in v0.1.0.}
}
\value{
A numeric vector of real values, in \code{base_year} prices.
}
\description{
Converts nominal values at year-of-occurrence prices to real values
at a chosen base year, using the bundled GDP deflator.
}
\examples{
gb_real(nominal_values = c(100, 110, 120),
        year = 2020:2022,
        base_year = 2024)
}
\references{
HM Treasury (2022). The Green Book, chapter on real
terms and price-base alignment.
}
\seealso{
\code{\link[=gb_deflator]{gb_deflator()}}, \code{\link[=gb_rebase]{gb_rebase()}}.

Other deflator: 
\code{\link{gb_deflator}()},
\code{\link{gb_rebase}()}
}
\concept{deflator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{gb_cost_per_unit}
\alias{gb_cost_per_unit}
\title{Cost per unit delivered}
\usage{
gb_cost_per_unit(appraisal, units_delivered, unit = "unit")
}
\arguments{
\item{appraisal}{A \code{gb_appraisal} object.}

\item{units_delivered}{Numeric scalar. The total quantity of the
output being delivered (e.g. QALYs gained, tCO2e abated,
WELLBYs added, jobs created).}

\item{unit}{Character. Label for the unit (used in print).}
}
\value{
Numeric scalar: PV cost / units delivered, in GBP per
unit.
}
\description{
Computes a cost-effectiveness ratio: total real cost (PV) per
unit of monetised or non-monetised output. Standard cross-scheme
comparator used by HM Treasury reviewers.
}
\examples{
app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 0, 0, 0, 0))
gb_cost_per_unit(app, units_delivered = 50, unit = "QALY")
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}.

Other reporting: 
\code{\link{gb_headline}()},
\code{\link{gb_to_excel}()},
\code{\link{gb_to_latex}()},
\code{\link{gb_to_word}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stpr.R
\name{gb_stpr}
\alias{gb_stpr}
\title{Social Time Preference Rate}
\usage{
gb_stpr(years, schedule = "standard")
}
\arguments{
\item{years}{Integer vector of years from the base year (year 0 is
the base year). Must be non-negative.}

\item{schedule}{One of \code{"standard"} (default, 3.5 percent baseline),
\code{"health"} (1.5 percent baseline, used in DHSC supplementary
guidance), or \code{"catastrophic"} (3.0 percent, for projects where
catastrophic risk dominates).}
}
\value{
A numeric vector of discount rates (decimals, e.g. 0.035 for
3.5 percent), one per element of \code{years}.
}
\description{
Returns the HM Treasury Green Book Social Time Preference Rate (STPR)
for a vector of years from a base year. The schedule is kinked: the
rate steps down at 30, 75, 125, 200, and 300 years. Three variants
are supported.
}
\details{
The STPR is composed of pure time preference plus a wealth-effect
adjustment for expected per-capita consumption growth. The kink
reflects increasing uncertainty over the constituent parameters at
longer horizons.
}
\examples{
gb_stpr(0:5)
gb_stpr(c(10, 30, 31, 75, 76))
gb_stpr(c(10, 30, 31, 75, 76), schedule = "health")
}
\references{
HM Treasury (2022). The Green Book: Central Government Guidance on
Appraisal and Evaluation, Annex A6.

HM Treasury (2003). Green Book Supplementary Guidance: Discounting.
}
\seealso{
\code{\link[=gb_discount_factor]{gb_discount_factor()}}, \code{\link[=gb_discount]{gb_discount()}}, \code{\link[=gb_npv]{gb_npv()}}.

Other discounting: 
\code{\link{gb_discount}()},
\code{\link{gb_discount_factor}()},
\code{\link{gb_eanc}()},
\code{\link{gb_npv}()}
}
\concept{discounting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lookups.R
\name{gb_schedule_table}
\alias{gb_schedule_table}
\title{Tibble of the STPR schedule}
\usage{
gb_schedule_table(schedule = NULL)
}
\arguments{
\item{schedule}{Optional. One of \code{"standard"}, \code{"health"},
\code{"catastrophic"}. If \code{NULL} (default), all three columns are
returned.}
}
\value{
A data frame.
}
\description{
Returns the bundled STPR kinked schedule as a data frame: one row
per band, columns \code{year_from}, \code{year_to}, and (depending on
\code{schedule}) either all three rate variants or just the requested
one.
}
\examples{
gb_schedule_table()
gb_schedule_table("health")
}
\seealso{
\code{\link[=gb_stpr]{gb_stpr()}}, \code{\link[=gb_data_versions]{gb_data_versions()}}.

Other lookups: 
\code{\link{gb_data_versions}()}
}
\concept{lookups}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/progression.R
\name{gb_progression}
\alias{gb_progression}
\title{Track an appraisal across SOC, OBC, and FBC stages}
\usage{
gb_progression(soc, obc = NULL, fbc = NULL)
}
\arguments{
\item{soc}{Strategic Outline Case \code{gb_appraisal}.}

\item{obc}{Outline Business Case \code{gb_appraisal}. Optional.}

\item{fbc}{Full Business Case \code{gb_appraisal}. Optional.}
}
\value{
A \code{gb_progression} object with elements \code{stages},
\code{evolution} (data frame), \code{delta_npv}, \code{delta_costs}.
}
\description{
Builds a \code{gb_progression} showing how NPV, BCR, and key
parameters evolve across the three Green Book business case
stages: Strategic Outline Case (SOC), Outline Business Case
(OBC), and Full Business Case (FBC). Useful for both authoring
a multi-stage appraisal and reviewing how figures have moved
between gateway approvals.
}
\details{
At each stage, optimism bias mitigation typically increases as
project definition firms up. Cost estimates converge towards
base-cost reality; benefit estimates may also shift as evidence
accumulates. The \code{evolution} table makes the trajectory visible.
}
\examples{
soc <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30),
                   ob = "non_standard_buildings", ob_mitigation = 0)
obc <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30),
                   ob = "non_standard_buildings", ob_mitigation = 0.5)
fbc <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30),
                   ob = "non_standard_buildings", ob_mitigation = 0.9)
gb_progression(soc, obc, fbc)
}
\references{
HM Treasury (2018). Guide to Developing the Project
Business Case (Green Book supplementary guidance).
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}.

Other appraisal: 
\code{\link{gb_appraise}()},
\code{\link{gb_compare}()},
\code{\link{gb_economic_case}()},
\code{\link{gb_place_based}()},
\code{\link{gb_risk_register}()},
\code{\link{gb_sensitivity_ob}()},
\code{\link{gb_validate}()}
}
\concept{appraisal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{gb_validate}
\alias{gb_validate}
\title{Lint a Green Book appraisal for common errors}
\usage{
gb_validate(appraisal)
}
\arguments{
\item{appraisal}{A \code{gb_appraisal} object.}
}
\value{
A \code{gb_validation} object with elements \code{pass}, \code{errors},
\code{warnings}, \code{info}, and \code{n_checks}.
}
\description{
Inspects a \code{gb_appraisal} for sign-convention errors, base-year
alignment, schedule plausibility, and other common authoring
mistakes. Returns a structured report classifying each check as
pass, warning, or error.
}
\details{
Checks performed:
\itemize{
\item Cashflow length matches \code{years} length
\item Schedule is one of the valid options
\item Base year is plausible (1900 to 2100)
\item Vintage matches a recognised Green Book edition
\item If \code{costs} and \code{benefits} are present (from \code{gb_appraise()}),
costs are non-negative and benefits are non-negative
\item NPV is finite (no NaN / Inf)
\item PV costs and PV benefits are consistent with NPV
}

Warnings flag suspicious patterns: missing base year, unusual
optimism bias values, METB outside 0 to 50 percent, very long
horizons (> 100 years).
}
\examples{
app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
gb_validate(app)
}
\seealso{
\code{\link[=gb_appraise]{gb_appraise()}}.

Other appraisal: 
\code{\link{gb_appraise}()},
\code{\link{gb_compare}()},
\code{\link{gb_economic_case}()},
\code{\link{gb_place_based}()},
\code{\link{gb_progression}()},
\code{\link{gb_risk_register}()},
\code{\link{gb_sensitivity_ob}()}
}
\concept{appraisal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimism_bias.R
\name{gb_optimism_bias}
\alias{gb_optimism_bias}
\title{Optimism bias upper bound for a project category}
\usage{
gb_optimism_bias(category, dimension = "capex")
}
\arguments{
\item{category}{Character. One of
\code{"standard_buildings"}, \code{"non_standard_buildings"},
\code{"standard_civil_engineering"},
\code{"non_standard_civil_engineering"},
\code{"equipment_development"}, \code{"outsourcing"}.}

\item{dimension}{Character. \code{"capex"} (default) for capital
expenditure uplift, or \code{"duration"} for works-duration uplift.}
}
\value{
Numeric scalar: the upper-bound percentage as a decimal
(e.g. 0.51 for 51 percent).
}
\description{
Returns the indicative upper-bound optimism bias percentage from
HM Treasury Supplementary Green Book Guidance: Optimism Bias
(Mott MacDonald 2002, Annex A1). The upper bound is the starting
value for ex-ante uplift; mitigation factors reduce it as project
definition matures through SOC, OBC, and FBC stages.
}
\examples{
gb_optimism_bias("non_standard_buildings")
gb_optimism_bias("standard_civil_engineering", dimension = "duration")
}
\references{
HM Treasury (2003). Supplementary Green Book Guidance: Optimism
Bias.

Mott MacDonald (2002). Review of Large Public Procurement in the
UK. Report commissioned by HM Treasury.
}
\seealso{
\code{\link[=gb_apply_ob]{gb_apply_ob()}}, \code{\link[=gb_categories]{gb_categories()}}, \code{\link[=gb_appraise]{gb_appraise()}}.

Other optimism bias: 
\code{\link{gb_apply_ob}()},
\code{\link{gb_categories}()}
}
\concept{optimism bias}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporting.R
\name{gb_to_word}
\alias{gb_to_word}
\title{Export an appraisal to Word}
\usage{
gb_to_word(appraisal, file)
}
\arguments{
\item{appraisal}{A \code{gb_appraisal} object.}

\item{file}{Output file path (must end in \code{.docx}).}
}
\value{
Invisibly, the file path.
}
\description{
Writes a one-page Word document with the appraisal headline and
cashflow table.
}
\details{
Requires the \code{officer} and \code{flextable} packages (both in Suggests).
}
\examples{
\donttest{
if (requireNamespace("officer", quietly = TRUE) &&
    requireNamespace("flextable", quietly = TRUE)) {
  app <- gb_appraise(c(100, 0, 0, 0, 0), c(0, 30, 30, 30, 30))
  tmp <- tempfile(fileext = ".docx")
  gb_to_word(app, tmp)
}
}
}
\seealso{
\code{\link[=gb_to_latex]{gb_to_latex()}}, \code{\link[=gb_to_excel]{gb_to_excel()}}.

Other reporting: 
\code{\link{gb_cost_per_unit}()},
\code{\link{gb_headline}()},
\code{\link{gb_to_excel}()},
\code{\link{gb_to_latex}()}
}
\concept{reporting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/valuation.R
\name{gb_vpf}
\alias{gb_vpf}
\title{Value of Preventing a Fatality}
\usage{
gb_vpf(year = 2024, series = "central")
}
\arguments{
\item{year}{Integer scalar. The year in which the value is
expressed. Default \code{2024} (the most recent DfT-published
value).}

\item{series}{Character. \code{"central"} (default). Reserved for
future expansion (DfT cancer / aversion multipliers).}
}
\value{
Numeric scalar: the VPF in GBP at year \code{year} prices.
}
\description{
Returns the DfT TAG-published Value of Preventing a Fatality
(VPF) in real terms for a given year. The published 2024 value
is GBP 2.153 million; the historical 2018 value (DfT TAG) is
GBP 1.958 million. Years between bundled publication dates are
filled by an annual real uplift of approximately 2 percent
(proxy for real GDP per head growth).
}
\examples{
gb_vpf()
gb_vpf(2018)
}
\references{
Department for Transport. Transport Analysis
Guidance (TAG) data book.
}
\seealso{
\code{\link[=gb_wellby]{gb_wellby()}}, \code{\link[=gb_qaly]{gb_qaly()}}.

Other valuation: 
\code{\link{gb_qaly}()},
\code{\link{gb_wellby}()}
}
\concept{valuation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimism_bias.R
\name{gb_apply_ob}
\alias{gb_apply_ob}
\title{Apply an optimism bias uplift to a cost stream}
\usage{
gb_apply_ob(values, ob_pct, mitigation = 0)
}
\arguments{
\item{values}{Numeric vector of baseline cost values.}

\item{ob_pct}{Numeric scalar. Optimism bias percentage as a
decimal. Pass \code{gb_optimism_bias(category)} to use the published
upper bound.}

\item{mitigation}{Numeric scalar in \verb{[0, 1]}. Fraction of the
upper bound that has been mitigated through project definition
and risk management. Default \code{0} (no mitigation; full upper
bound applied).}
}
\value{
A numeric vector the same length as \code{values}, with the
uplift applied.
}
\description{
Applies an OB uplift, optionally with a mitigation factor that
represents progress on risk identification and management:
\code{cost_with_ob = cost_baseline * (1 + ob_pct * (1 - mitigation))}.
}
\examples{
costs <- c(100, 50, 50)
ob <- gb_optimism_bias("non_standard_buildings")
gb_apply_ob(costs, ob)
gb_apply_ob(costs, ob, mitigation = 0.5)
}
\references{
HM Treasury (2003). Supplementary Green Book Guidance:
Optimism Bias, Annex A2 on mitigation factors.
}
\seealso{
\code{\link[=gb_optimism_bias]{gb_optimism_bias()}}, \code{\link[=gb_appraise]{gb_appraise()}}.

Other optimism bias: 
\code{\link{gb_categories}()},
\code{\link{gb_optimism_bias}()}
}
\concept{optimism bias}
